/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#ifndef _GUI_H_
#define _GUI_H_
#include <vector>
#include <string>
#include "thEngine.h"
#include "xmlshader.h"

#ifdef CreateWindow
#undef CreateWindow
#endif

bool XmlLoadColor(TiXmlNode *pChild,vec4& Color);

struct ThRect {
  int x,y,width,height;
  float fx1,fy1,fx2,fy2;
};

struct ThWindowStyle {
  ThRect lo,lu,ro,ru,lm,rm,l,r,o,u,fill,bar;
  int CaptionX,CaptionY,CaptionFont;
  vec4 CaptionColor;
};

struct ThButtonStyle {
  ThRect lo,lu,ro,ru,l,r,o,u,fill;
  vec4 Color,CaptionColor;
  int CaptionFont;
};

struct ThElementStyle {
  ThRect fill;
  vec4 Color;
};

struct ThSliderStyle {
  ThRect top,center,bottom;
  vec4 Color,CaptionColor;
  int CaptionFont;
};

struct ThRectStyle {
  ThRect lo,lu,ro,ru,l,r,o,u,fill;
  vec4 Color,CaptionColor;
  int CaptionFont,CaptionX,CaptionY;
};

struct ThGuiFont {
  int num;
  float x,y;
  vec4 Color;
  std::string Text;
};

struct ThGuiElement {
  int iOrder,iRectCount,iTextCount;
};

struct ThGuiSelectInit {
  int Style;
  int SelectStyle,SelectHoverStyle,SelectSelectedStyle;
  int ButtonUpStyle,ButtonUpHoverStyle;
  int ButtonDownStyle,ButtonDownHoverStyle;
  int ScrollStyle,ScrollHoverStyle;
  int Space;
  int MaxElementsVisible;
  int Width;
};

typedef void (*GuiEvent)(void*);

class Gui {
  private:
    std::vector<ThWindowStyle> WindowStyles;
    std::vector<ThButtonStyle> ButtonStyles;
    std::vector<ThElementStyle> ElementStyles;
    std::vector<ThSliderStyle> SliderStyles;
    std::vector<ThRectStyle> RectStyles;
    std::vector<class GuiEventRect*> Childs;
    std::vector<ThGuiFont> FontDrawCalls;
    std::vector<ThGuiElement> GuiElements;
    int iRectCount,iTextCount;
    Texture2D *GuiTexture;
    VertexBuffer *vbuffer;
    XMLShader *GuiShader;
    class GuiEventRect *TextReciever;

    bool ComputeTexCoords(ThRect& Rect);

  public:
    Gui();
    ~Gui();
    bool LoadStyles(const char *filename);
    bool LoadGuiTexture(const char *filename);
    bool LoadGuiShader(const char *filename);
    void DrawRect(ThRect Rect, vec4 Color);
    void DrawFont(int set, int x, int y, vec4 Color, const char *Text);
    void FinishElement();
    ThButtonStyle* GetButtonStyle(int num);
    ThWindowStyle* GetWindowStyle(int num);
    ThElementStyle* GetElementStyle(int num);
    ThSliderStyle* GetSliderStyle(int num);
    ThRectStyle* GetRectStyle(int num);
    void AddChild(class GuiEventRect* Child);
    void Draw();

    void MouseDown(int button, int x, int y);
    void MouseUp(int button, int x, int y);
    void MouseMove(int xabs, int yabs, int xrel, int yrel);
    void TextIn(char);

    void SetTextReciever(class GuiEventRect* ptr){ TextReciever = ptr; }
    const GuiEventRect* GetTextReciever() const {return TextReciever;}

    class GuiWindow* CreateWindow(class GuiEventRect* Father,int x, int y, int width, int height,bool Fixed, int Style, const char *Caption, void* Creator);
    class GuiButton* CreateButton(class GuiEventRect* Father,int x, int y, int width, int height, int NormalStyle, int HoverStyle, const char *Caption, void* Creator);
    class GuiTextInput* CreateTextInput(class GuiEventRect* Father, int x, int y, int width, int height, int NormalStyle, int ActiveStyle, int MaxLength, const char *Caption, void* Creator);
    class GuiSliderVertical* CreateSliderVertical(class GuiEventRect *Father, int x, int y, int Height, float Min, float Max, int Style, int ChildStyle, int ChildHoverStyle, const char *Caption, void* Creator);
    class GuiColorView* CreateColorView(class GuiEventRect* Father, int x, int y, int Style, void* Creator);
    class GuiSelect* CreateSelect(class GuiEventRect* Father, int x, int y, ThGuiSelectInit InitStruct, void* Creator);
    class GuiText* CreateText(class GuiEventRect *Father, int x, int y, int Font, const char *text, void* Creator);
};

class GuiEventRect {
  protected:
    int ix,iy,iwidth,iheight;
    std::vector<class GuiEventRect*> Childs;
    bool active,bMouseIn,bFocus;
    Gui *theGui;
    void *Creator;

  public:
    GuiEventRect(Gui* pGui);
    ~GuiEventRect();
    virtual bool MouseDown(class GuiEventRect *Father,int button, int x, int y);
    virtual bool MouseUp(class GuiEventRect *Fahter,int button, int x, int y);
    virtual bool MouseMove(class GuiEventRect *Father, int xabs, int yabs,int xrel, int yrel);
    virtual void MouseOut();
    virtual void LooseFocus();
    virtual void TextIn(char text);
    virtual void Draw(int x, int y)=0;
    void AddChild(class GuiEventRect *Child);
    void SetActive(bool value){active = value;}
    void SetRect(int x, int y, int width, int height){ix = x;iy=y;iwidth=width;iheight=height;}
    void SetPosition(int x, int y){ix = x; iy=y;}
    void SetCreator(void* ptr){Creator = ptr;}
    inline int GetX() const {return ix;}
    inline int GetY() const {return iy;}
    inline int GetWidth() const {return iwidth;}
    inline int GetHeight() const {return iheight;}
};

class GuiWindow : public GuiEventRect {
  private:
    bool bFixed,bDraging;
    int iStyle;
    std::string Caption;

  public:
    GuiWindow(Gui* pGui,bool fixed, int Style, const char *pCaption);
    bool MouseDown(class GuiEventRect *Father,int button, int x, int y);
    bool MouseUp(class GuiEventRect *Fahter,int button, int x, int y);
    bool MouseMove(class GuiEventRect *Father, int xabs, int yabs,int xrel, int yrel);
    void MouseOut();
    void Draw(int x, int y);
};

class GuiButton : public GuiEventRect {
  private:
    int iNormalStyle,iHoverStyle;
    std::string Caption;
    GuiEvent OnPress;

  public:
    GuiButton(Gui* pGui,int NormalStyle, int HoverStyle, const char *pCaption);
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
    bool MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel);
    void MouseOut();
    void Draw(int x, int y);
    void SetOnPress(GuiEvent Func){OnPress = Func;}
    void SetCaption(const char *pCaption){Caption = pCaption;}
};

class GuiTextInput : public GuiEventRect {
  private:
    int iNormalStyle, iActiveStyle, iMaxLength;
    std::string Caption;
    GuiEvent OnChange;
    GuiEvent OnLooseFocus;

  public:
    GuiTextInput(Gui* pGui,int NormalStyle, int ActiveStyle, int MaxLength, const char *pCaption);
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
    void LooseFocus();
    void Draw(int x, int y);
    void SetOnChange(GuiEvent Func){OnChange = Func;}
    void SetOnLooseFocus(GuiEvent Func){OnLooseFocus = Func;}
    void TextIn(char text);
    void SetCaption(const char *pCaption);
    inline const std::string& GetCaption(){return Caption;}
};

class GuiActionElement : public GuiEventRect {
  protected:
    int iNormalStyle, iHoverStyle;

  public:
    GuiActionElement(Gui* pGui, int NormalStyle, int HoverStyle);
    virtual bool MouseDown(class GuiEventRect *Father, int button, int x, int y)=0;
    virtual bool MouseUp(class GuiEventRect *Father, int button, int x, int y)=0;
    virtual bool MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel);
    virtual void MouseOut();
    void Draw(int x, int y);
};

class GuiSliderVertical : public GuiEventRect {
  private:
    float fMax,fMin,fValue;
    std::string Caption;
    GuiEvent OnChange;
    int iSliderHeight,iTextHeight,iTextWidth,iStyle;
    int iChildStyle;

  public:
    GuiSliderVertical(Gui *pGui, int Height, float Min, float Max, int Style, int ChildStyle, int ChildHoverStyle, const char *pCaption);
    bool MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel);
    void Draw(int x, int y);
    void SetOnChange(GuiEvent Func){OnChange = Func;}
    inline float GetValue() const {return fValue;}
    void SetValue(float value);
    void SetMinMax(float Min, float Max){fMin = Min; fMax = Max;}
};

class GuiSliderVerticalChild : public GuiActionElement {
  private:
    int iMax,iMin;
    bool bDraging;

  public:
    GuiSliderVerticalChild(Gui *pGui, int NormalStyle, int HoverStyle,int x, int y, int Min, int Max);
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
    bool MouseUp(class GuiEventRect *Father, int button, int x, int y);
    bool MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel);
    void MouseOut();
};

class GuiColorView : public GuiEventRect {
  private:
    vec4 Color;
    int iStyle;
  public:
    GuiColorView(Gui* pGui, int Style);
    void SetColor(vec4 value){Color = value;}
    void SetRed(float value){Color.x = value;}
    void SetGreen(float value){Color.y = value;}
    void SetBlue(float value){Color.z = value;}
    void SetAlpha(float value){Color.w = value;}
    vec4 GetColor() const {return Color;}
    void Draw(int x, int y);
};

class GuiSelectButton : public GuiActionElement {
  public:
    GuiSelectButton(Gui* pGui, int NormalStyle, int HoverStyle);
    bool MouseUp(class GuiEventRect* Father, int button, int x, int y);
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
};

class GuiSelectElement : public GuiEventRect {
  private:
    int iNormalStyle,iHoverStyle,iSelectedStyle;
    bool bSelected;
    std::string Caption;

  public:
    GuiSelectElement(Gui* pGui, int NormalStyle, int HoverStyle, int SelectedStyle);
    void SetCaption(const char *pCaption){Caption = pCaption;}
    bool MouseUp(class GuiEventRect *Father, int button, int x, int y);
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
    void Draw(int x, int y);
};

class GuiSelect : public GuiEventRect {
  private:
    GuiEventRect *Up,*Down,*ScrollBar;
    int iStyle,iSelected,iShowFrom,iShowAnz,iElementWidth,iSpace;
    int iSelectStyle,iSelectHoverStyle,iSelectSelectedStyle;
    GuiEvent OnChange;

    void Update();

  public:
    GuiSelect(Gui* pGui, ThGuiSelectInit InitStruct);
    ~GuiSelect();
    bool MouseDown(class GuiEventRect *Father, int button, int x, int y);
    bool MouseUp(class GuiEventRect *Father, int button, int x, int y);
    bool MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel);
    void Draw(int x, int y);
    void AddOption(const char* Caption);
    void InsertOption(int insert_pos,const char* Caption);
    void RemoveOption(int num);
    void RemoveAllOptions();
    void ChangeOption(int num, const char *Caption);
    void SetOnChange(GuiEvent Func){OnChange = Func;}
    inline int GetSelected() const {return iSelected;}
};

class GuiText : public GuiEventRect {
  private:
    std::string Caption;
    int iFont;
    vec4 Color;

  public:
    GuiText(Gui* pGui,int Font,const char *pCaption) : GuiEventRect(pGui) {Caption = pCaption; iFont = Font; Color.set(1.0f);}
    void Draw(int x, int y){
      if(!active)
        return;
      x+=ix;
      y+=iy;
      if(!Caption.empty())
        theGui->DrawFont(iFont,x,y,Color,Caption.c_str());
    }
    void SetColor(vec4 value){Color = value;}
    void SetCaption(const char *str){Caption = str;}
    void PrintToCaption(const char *str,...);
};
#endif
