/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#ifndef _PLANTS_H_
#define _PLANTS_H_
#include "thEngine.h"
#include "object.h"
#include "shader.h"
#include "model.h"
#include "landscape.h"
#include "fbo.h"
#include "xmlshader.h"
#include "instancing.h"
#include <vector>
#include <stack>
#include <algorithm>

class GrasSort {
  public:
    int num;
    float fDistance;

    inline bool operator < (const GrasSort& r) const{
      return (this->fDistance < r.fDistance);
    }

    inline bool operator > (const GrasSort& r) const{
      return (this->fDistance > r.fDistance);
    }
};

class PlantScrub : public Object{
  private:
    static Shader *ColorShader;
    static Shader *ShadowShader;
    Texture2D *Textur;
    GLuint gluiVertexBuffer;
    char *lpcVertexData;
    char *lpcTexCoordData1;
    char *lpcTexCoordData2;
    char *lpcNormalData;
    int NumLeafes;
    float fRadius;
    float fTime;

  public:
    PlantScrub(float Radius, int RandomSeed, Texture2D *ptr);
    ~PlantScrub();
    void Draw(char mode);
    void DrawBounding();
    void Recalc();
    void SetTime(float value);
    static void SetColorShader(Shader *ptr);
    static void SetShadowShader(Shader *ptr);
};

class PlantGrasQuad : public Object {
  protected:
    std::vector<vec4> v4Positions;
    std::vector<vec4> v4Normals;
    std::vector<GrasSort> SortedList;
    std::stack<int> UnsortedList;
    static float fGrasDistance;
    static vec4 CamPos;

  public:
    PlantGrasQuad(Landscape *Terrain, float x, float y, float size, Texture2D *coverage, vec4 TargetColor);
    float Distance();
    void Sort(vec4& CamPos);
    static void SetGrasDistance(float value){fGrasDistance = value;}
    static void SetCamPos(vec4 value){CamPos = value;}
};

class PlantGrasQuadMesh : public PlantGrasQuad {
  private:
    std::vector<mat4a> Matrizen;
    InstancedVertexBuffer *InstancedVBuffer;
    int iInstanceGroup;

  public:
    PlantGrasQuadMesh(InstancedVertexBuffer* vbuffer,Landscape *Terrain, float x, float y, float size, Texture2D *coverage, vec4 TargetColor);
    void Draw(char mode);
};

class PlantManager {
  protected:
    int iDivisions;
    Landscape *Terrain;
    std::vector<InstancedVertexBuffer*> Models;
    std::vector<XMLShader*> Shader;
    FrustrumObjectBox *FrustrumObjects;
    std::vector<Texture2D*> Textures;
    std::vector<int> ModelTextureUsage;
    std::vector<int> ModelShaderUsage;
    std::vector<vec4> Colors;
    float fGrasDistance;
    int iGetShadow;
    FrameBufferTexture2D *DepthTexture;
    bool *FrustrumVisible;
    static bool bFallbackInstancing;

  public:
    PlantManager(Landscape *ter, int Divisions);
    ~PlantManager();
    void AddTexture(Texture2D *ptr);
    void AddShader(XMLShader *ptr);
    void AddModel(Model *ptr, vec4 Color, int Texture, int ShaderNum);
    void SetGetShadow(int value){iGetShadow = value;}
    void SetDepthTexture(FrameBufferTexture2D *ptr){DepthTexture = ptr;}
    virtual void SetGrasDistance(float value)=0;
    virtual void Generate(Texture2D *coverage)=0;
    virtual void Draw(vec4 CamPos,vec4 CamShadowMapPos, float fTime)=0;
    static void SetFallbackInstancing(bool value);
};

class PlantManagerSoft : public PlantManager{
  private:
    PlantGrasQuadMesh **Plants;

  public:
    PlantManagerSoft(Landscape *ter, int Divisions);
    ~PlantManagerSoft();
    void SetGrasDistance(float value);
    void Generate(Texture2D *coverage);
    void Draw(vec4 CamPos,vec4 CamShadowMapPos, float fTime);
};

struct TreeInfo {
  VertexBuffer *vbuffer;
  int ColorShader;
  int ShadowShader;
  int Texture;
  int BumpTexture;
  int ChildOf;
  int ColorOptions;
  int ShadowOptions;
  bool bTextureInShadowNeeded;
  vec4 Color,min,max;
  int Count;
  mat4a *ModelMatrices;
  FrustrumObjectBox *Frustrums;
  bool* bVisible;
};

class TreeManager {
  private:
    std::vector<TreeInfo> Models;
    std::vector<Shader*> Shaders;
    std::vector<Texture2D*> Textures;

  public:
   ~TreeManager();
   int AddShader(Shader *ptr);
   int AddTexture(Texture2D *ptr);
   int AddModel(TreeInfo* ptr);
   void Generate(Texture2D *coverage,Landscape *Terrain);
   void Draw(bool Color);
   void DrawBoxes();
};

#endif
