/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#ifndef _SHADER_H_
#define _SHADER_H_
#include "thtypes.h"
#include <vector>
#define TH_FLEXIBLE_SHADERS

#define TH_VERTEX_SHADER   0
#define TH_GEOMETRY_SHADER 1
#define TH_FRAGMENT_SHADER 2

#define TH_GS_INPUT        0
#define TH_GS_OUTPUT       1
#define TH_GS_OUTPUT_NUM   2

void UnloadShader();

class ShaderConstant {
  public:
    virtual void Update(int num,class Shader *ptr)=0;
    virtual void Set(const void *ptr)=0;
};

class ShaderConstantFloat : public ShaderConstant {
  private:
    float data;
  public:
    void Update(int num,class Shader *ptr);
    void Set(const void *ptr);
};

class ShaderConstantVec3 : public ShaderConstant {
  private:
    vec3 data;
  public:
    void Update(int num, class Shader *ptr);
    void Set(const void *ptr);
};

class ShaderConstantVec4 : public ShaderConstant {
  private:
    vec4 data;
  public:
    void Update(int num,class Shader *ptr);
    void Set(const void *ptr);
};

class ShaderConstantMat4a : public ShaderConstant {
  private:
    mat4a data;
  public:
    void Update(int num,class Shader *ptr);
    void Set(const void *ptr);
};

struct ShaderConstantReference {
  int num;
  ShaderConstant *ptr;
};

class Shader {
  private:
    GLhandleARB glhShaderProgram;
    GLhandleARB glhVertexShader;
    GLhandleARB glhGeometryShader;
    GLhandleARB glhFragmentShader;
    bool linked;
    void getError(GLhandleARB obj);
    bool CheckWarnings(GLhandleARB obj, const char *title);
    char *cName;
    GLint *gliUniforms;
    GLuint *gluiUniformBuffers;
    GLint *gliAttribs;
    int iAttribCount;
    int iUniformCount;
    int iInstanceUniform;
    int iInstanceAttribute;
    std::vector<ShaderConstantReference> ShaderConstants;

    bool ShaderSource(const char *source, GLhandleARB& glhShader, GLenum ShaderType);

  public:
    Shader(const char *name);
    ~Shader();
    bool LoadVertexShader(const char * lpcFileName);
    bool LoadGeometryShader(const char *lpcFileName);
    void SetGSParameter(char option, int value);
    bool LoadFragmentShader(const char *lpcFileName);
    bool LoadShaderSource(const char *source, int type);
    bool Validate();
    bool Link();
    void Use();
    inline char *GetName() const {return cName;}
    inline int GetAttribCount() const {return iAttribCount;}
    inline int GetUniformCount() const {return iUniformCount;}
    GLint GetAttrib(const char* name);
    GLint GetUniform(const char* name);
    bool LoadUniforms(const char* names);
    bool LoadAttributes(const char* names);
    GLint GetAttributeLocation(int num);
    //Float 1-4
    void SetUniform(int num,float f1);
    void SetUniform(int num,float f1,float f2);
    void SetUniform(int num,float f1,float f2,float f3);
    void SetUniform(int num,float f1,float f2,float f3,float f4);
    //Float Array
    void SetUniform(int num,int size1,int size2,float *ptr);
    void SetUniform(int num,int size,bool transponse,float *array);
    void SetUniform(int num,vec4 vektor);
    void SetUniform(int num,vec3 vektor);
    //Int 1-4
    void SetUniform(int num,int i1);
    void SetUniform(int num,int i1,int i2);
    void SetUniform(int num,int i1,int i2,int i3);
    void SetUniform(int num,int i1,int i2,int i3,int i4);

    void BindBufferToUniform(int num, GLuint Buffer);
    bool GenBuffer(int num, int size, GLenum mode);
    bool UploadData(int num, int start, int size, const void *ptr);

    void SetInstanceAttribute(int value){iInstanceAttribute = value;}
    void SetInstanceUniform(int value){iInstanceUniform = value;}
    inline int GetInstanceAttribute() const {return iInstanceAttribute;}
    inline int GetInstanceUniform() const {return iInstanceUniform;}

    void AddShaderConstant(int num, ShaderConstant *ptr);
};
#endif
