/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#ifndef _VEKTOREN_H_
#define _VEKTOREN_H_
#include "my3d.h"
#include <math.h>

#ifdef __SSE__
#include "sse.h"
#define ALIGNED16 __attribute__ ((aligned(16)));
#else
#define ALIGNED16
#endif

#ifdef __SSE__
//Memalign fehlt also makro
#ifdef WIN32
#include <malloc.h>
#define th_memalign(a,s) __mingw_aligned_malloc(s,a)
#define th_free(p) __mingw_aligned_free(p)
#elif defined __linux
#include <malloc.h>
#define th_memalign(a,s) memalign(a,s);
#define th_free(p) free(p);
#endif
#endif


//Vektor und Matrizen Datenstrukturen

typedef union {
  struct {
    float x;
    float y;
    float z;
  };
  float f[3];
}vec3;

struct mat4{
  float f[16];
};

struct mat3a{
  float f[9];
};

typedef struct {
  float x;
  float y;
}vec2;

//Vektor Klasse
class vec4{
  public:
    #ifdef __SSE__
    //berladene new und delete
    static void* operator new (size_t size);
    static void* operator new[] (size_t size);
    static void operator delete (void *pMem);
    static void operator delete[] (void *pMem);

    union
    {
      struct {
        float x;
        float y;
        float z;
        float w;
      };
      float f[4];
      __m128 m;
    } __attribute__ ((aligned(16)));
    #else
    union {
      struct {
        float x;
        float y;
        float z;
        float w;
      };
      float f[4];
    };
    #endif

    //Konstruktoren
    vec4() {};
    vec4(const float fx, const float fy, const float fz, const float fw) {x=fx;y=fy;z=fz;w=fw;}

    //
    inline void set(float f){x=f;y=f;z=f;w=f;}

    //Operatoren
    //Unary operatoren
    vec4 operator + () const { return *this; }
    vec4 operator - () const { return vec4(-x,-y,-z,-w);}

    //Rechenoperatoren
    inline vec4 operator + (const vec4& v) const{
      vec4 res;
      res.x = this->x + v.x;
      res.y = this->y + v.y;
      res.z = this->z + v.z;
      res.w = this->w + v.w;
      return res;
    }

    inline vec4 operator - (const vec4& v) const{
      vec4 res;
      res.x = this->x - v.x;
      res.y = this->y - v.y;
      res.z = this->z - v.z;
      res.w = this->w - v.w;
      return res;
    }

    inline vec4 operator * (const vec4& v) const{
      vec4 res;
      res.x = this->x * v.x;
      res.y = this->y * v.y;
      res.z = this->z * v.z;
      res.w = this->w * v.w;
      return res;
    }

    inline vec4 operator *(const float& f) const{
      vec4 res;
      res.x = this->x * f;
      res.y = this->y * f;
      res.z = this->z * f;
      res.w = this->w * f;
      return res;
    }

    inline vec4 operator / (const vec4& v) const{
      vec4 res;
      res.x = this->x / v.x;
      res.y = this->y / v.y;
      res.z = this->z / v.z;
      res.w = this->w / v.w;
      return res;
    }

    inline vec4 operator / (const float& f) const{
      vec4 res;
      res.x = this->x / f;
      res.y = this->y / f;
      res.z = this->z / f;
      res.w = this->w / f;
      return res;
    }

    inline bool operator != (const vec4& v) const {
      return (this->x != v.x) || (this->y != v.y) || (this->z != v.z) || (this->w != v.w);
    }

    inline bool operator == (const vec4& v) const {
      return (this->x == v.x) && (this->y == v.y) && (this->z == v.z) && (this->w == v.w);
    }
};

//Matrix Klasse
class mat4a {
  public:
    #ifdef __SSE__
    static void* operator new (size_t size);
    static void* operator new[] (size_t size);
    static void operator delete (void *pMem);
    static void operator delete[] (void *pMem);
    union {
      float f[16];
      __m128 m[4];
    } __attribute__ ((aligned(16)));
    #else
    float f[16];
    #endif

    mat4a() {};

    mat4a operator * (const mat4a& m) const;
    vec4 operator * (const vec4& v) const;
    inline void set(float value){
      for(int i=0;i<16;i++)
        f[i]=value;
    };
};

vec4 DirectionV4(const vec4& a, const vec4& b);
vec4 ExtrudeV4(const vec4& v1, const vec4& v2, const float& fDistance);
vec4 ComputeNormalV4(const vec4& t1, const vec4& t2, const vec4& t3);
vec4 UpVektorV4(float X,float Y,float Z,float Radians);
vec4 CrossProductV4(const vec4& v1, const vec4& v2);

mat4a AlignMatrix4(float *matrix);
void DealignMatrix4(const mat4a& src, float *dst);
vec4 Mul(const mat4a& matrix,const vec4& vektor);
mat4a Mul(const mat4a& m2, const mat4a& m1);
const mat4a RotationMatrixXYZ(const vec3& v3Rotation);
const mat4a RotationMatrixXYZ(float x, float y, float z);
const mat4a TranslationMatrix(const vec4& Translation);
const mat4a TranslationMatrix(float x, float y, float z);
const mat4a ScaleMatrix(const vec3& v3Scale);
const mat4a ScaleMatrix(float x, float y, float z);

mat3a IdentityM3();
mat4a IdentityM4();
float Det(const mat3a& mat);
mat3a Inverse(const mat3a& ma);
mat3a Submat( const mat4a& mr, int i, int j );
float Det( const mat4a& mr );
mat4a Inverse( const mat4a& ma );
mat4a Transponse(const mat4a& m);
mat4a Right2Left(const mat4a& m);
mat4a NormalMatrix(const mat4a& m);

vec4 TangentV4(vec4 v1, vec4 v2, vec4 v3, vec2 t1, vec2 t2, vec2 t3);
vec4 BinormalV4(vec4 v1, vec4 v2, vec4 v3, vec2 t1, vec2 t2, vec2 t3);

vec4 NormalizeQ(const vec4& q);
vec4 CreateRotationQ(const vec4& axis, float angle);
vec4 MulQ(const vec4& q1, const vec4& q2);
vec4 InverseQ(const vec4& q);
mat4a QToMat4a(const vec4& q);

float DistancePlaneV4(const vec4& Plane,const vec4& v);
vec4 NormalizePlaneV4(const vec4& Plane);
vec4 PlanePoint(const vec4& p1, const vec4& p2, const vec4& p3);

vertex UpVektor(float X,float Y,float Z,float Radians);
vertex normalize(vertex vektor);

//Inline Functions
#ifdef __SSE__
inline vec4 AddV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.m = _mm_add_ps(v1.m,v2.m);
  return res;
}
#else
inline vec4 AddV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.x = v1.x + v2.x;
  res.y = v1.y + v2.y;
  res.z = v1.z + v2.z;
  res.w = v1.w + v2.w;
  return res;
}
#endif

#ifdef __SSE__
inline vec4 SubV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.m = _mm_sub_ps(v1.m,v2.m);
  return res;
}
#else
inline vec4 SubV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.x = v1.x - v2.x;
  res.y = v1.y - v2.y;
  res.z = v1.z - v2.z;
  res.w = v1.w - v2.w;
  return res;
}
#endif

#ifdef __SSE__
inline vec4 MulV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.m = _mm_mul_ps(v1.m,v2.m);
  return res;
}
#else
inline vec4 MulV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.x = v1.x * v2.x;
  res.y = v1.y * v2.y;
  res.z = v1.z * v2.z;
  res.w = v1.w * v2.w;
  return res;
}
#endif

#ifdef __SSE__
inline vec4 MulV4(const vec4& v1, const float value){
  vec4 temp,res;
  temp.m = _mm_set_ps(value,value,value,value);
  res.m = _mm_mul_ps(v1.m,temp.m);
  return res;
}
#else
inline vec4 MulV4(const vec4& v1, const float value){
  vec4 res;
  res.x = v1.x * value;
  res.y = v1.y * value;
  res.z = v1.z * value;
  res.w = v1.w * value;
  return res;
}
#endif

#ifdef __SSE__
inline vec4 DivV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.m = _mm_div_ps(v1.m,v2.m);
  return res;
}
#else
inline vec4 DivV4(const vec4& v1, const vec4& v2){
  vec4 res;
  res.x = v1.x / v2.x;
  res.y = v1.y / v2.y;
  res.z = v1.z / v2.z;
  res.w = v1.w / v2.w;
  return res;
}
#endif

#ifdef __SSE__
inline vec4 DivV4(const vec4& v1, const float& value){
  vec4 temp;
  temp.m = _mm_set_ps(value,value,value,value);
  temp.m = _mm_div_ps(v1.m,temp.m);
  return temp;
}
#else
inline vec4 DivV4(const vec4& v1, const float& value){
  vec4 res;
  res.x = v1.x / value;
  res.y = v1.y / value;
  res.z = v1.z / value;
  res.w = v1.w / value;
  return res;
}
#endif

///////////////// NORMALIZE V4 ///////////////////
#ifdef __SSE__
inline vec4 NormalizeV4(const vec4& vektor){
  float length;
  vec4 temp,res=vektor;
  temp=vektor * vektor;
  length=sqrt(temp.f[0]+temp.f[1]+temp.f[2]);
  if(length != 0){
    temp.set(length);
    res=vektor / temp;
  }
  return res;
}
#else
inline vec4 NormalizeV4(const vec4& vektor){
  float length;
  vec4 res=vektor;
  length=sqrt(vektor.x*vektor.x+vektor.y*vektor.y+vektor.z*vektor.z);
  if(length != 0){
    res.x=vektor.x/length;
    res.y=vektor.y/length;
    res.z=vektor.z/length;
  }
  return res;
}
#endif

inline float DistanceV4(const vec4& v1, const vec4& v2){
  vec4 temp;
  temp = v1 - v2;
  temp = temp * temp;
  return sqrt(temp.x + temp.y + temp.z);
}

/////////// Skalarproduct V4 //////////////
inline float SkalarProductV4(const vec4& a, const vec4& b){
  return a.x * b.x + a.y * b.y + a.z * b.z;
}

#endif
