/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/

#include "camera.h"
#include <math.h>
#include "vektoren.h"
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

camera::camera(){
  Pos1.x=0; Pos1.y=0; Pos1.z=0; Pos1.w=1;
  Pos2.x=1; Pos2.y=0; Pos2.z=0; Pos2.w=1;
  ReCalc=true;
  CameraMatrix = new float[16];
}

camera::~camera(){
  delete []CameraMatrix;
}

void camera::SetRotation(float Degrees){
  if(Degrees < 0)
    Degrees = 360 + Degrees;
  while(Degrees > 360)
    Degrees-=360;
  Rotation = Degrees / 180 * M_PI;
  ReCalc=true;
}

float camera::GetRotation(){
  return Rotation / M_PI * 180;
}

void camera::SetFrom(float x, float y, float z){
  Pos1.x = x;
  Pos1.y = y;
  Pos1.z = z;
  ReCalc=true;
}

void camera::SetFrom(vec4 vFrom){
  Pos1 = vFrom;
  ReCalc=true;
}

void camera::SetTo(float x, float y, float z){
  Pos2.x = x;
  Pos2.y = y;
  Pos2.z = z;
  ReCalc=true;
}

void camera::SetTo(vec4 vTo){
  Pos2 = vTo;
  ReCalc=true;
}

void camera::ChangeFrom(float x, float y, float z){
  Pos1.x += x;
  Pos1.y += y;
  Pos1.z += z;
  ReCalc=true;
}

void camera::ChangeTo(float x,float y, float z){
  Pos2.x += x;
  Pos2.y += y;
  Pos2.z += z;
  ReCalc=true;
}

void camera::Move(float x, float y, float z){
  Pos1.x+=x; Pos1.y+=y; Pos1.z+=z;
  Pos2.x+=x; Pos2.y+=y; Pos2.z+=z;
  ReCalc=true;
}

void camera::Set(float x, float y, float z){
  float DiffX,DiffY,DiffZ;
  DiffX = x - Pos1.x;
  DiffY = y - Pos1.y;
  DiffZ = z - Pos1.z;
  Pos1.x = x;      Pos1.y = y;      Pos1.z = z;
  Pos2.x += DiffX; Pos2.y *= DiffY; Pos2.z += DiffZ;
  ReCalc=true;
}

void camera::Call(){
  vec4 temp,dir,q;
  if(ReCalc){
    dir = DirectionV4(Pos1,Pos2);
    Up.x = 0; Up.y = 0; Up.z = 1.0f; Up.w=1.0f;
    if(SkalarProductV4(Up,dir)==1){
      Up.x = 1.0f;
      Up.y = 0.0f;
      Up.z = 0.0f;
    }
    else{
      temp = CrossProductV4(Up,dir);
      temp = NormalizeV4(temp);
      Up = CrossProductV4(dir,temp);
    }
    //Quaterion Zeugs
    if(Rotation != 0){
      q = CreateRotationQ(dir,Rotation);
      q = NormalizeQ(q);
      Up = Mul(QToMat4a(q),Up);
    }
  }
  gluLookAt(Pos1.x,Pos1.y,Pos1.z,Pos2.x,Pos2.y,Pos2.z,Up.x,Up.y,Up.z);
  glGetFloatv(GL_MODELVIEW_MATRIX, CameraMatrix);
}

vec4 camera::GetFrom(){
  return Pos1;
}

vec4 camera::GetTo(){
  return Pos2;
}

vec4 camera::GetUp(){
  return Up;
}

float* camera::GetCameraMatrix(){
  return CameraMatrix;
}

vec4 camera::GetViewDir(){
  return DirectionV4(Pos1,Pos2);
}

void camera::SaveToFile(const char *filename){
  FILE *datei;
  int i=0;
  datei = fopen(filename,"wb");
  if(!datei)
    return;
  fwrite(&Pos1,sizeof(vec4),1,datei);
  fwrite(&Pos2,sizeof(vec4),1,datei);
  fwrite(&Up,sizeof(vec4),1,datei);
  fwrite(&Rotation,sizeof(float),1,datei);
  fclose(datei);
}

void camera::LoadFromFile(const char *filename){
  FILE *datei;
  int i;
  datei = fopen(filename,"rb");
  if(!datei)
    return;
  fread(&Pos1,sizeof(vec4),1,datei);
  fread(&Pos2,sizeof(vec4),1,datei);
  fread(&Up,sizeof(vec4),1,datei);
  fread(&Rotation,sizeof(float),1,datei);
  fclose(datei);
}
