/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#include "gui.h"
#include "tinyxml.h"
#include <sstream>
#ifdef __linux
#include <stdarg.h>
#endif

Gui::Gui(){
  GuiTexture = NULL;
  GuiShader = NULL;
  vbuffer = new VertexBuffer(TH_VB_COLOR | TH_VB_TEXCOORD0_2 | TH_VB_DYNAMIC, GL_QUADS, GL_UNSIGNED_INT);
  TextReciever = NULL;
}

Gui::~Gui(){
  if(GuiTexture != NULL)
    delete GuiTexture;
  if(GuiShader != NULL)
    delete GuiShader;
  delete vbuffer;
  for(int i=0;i<Childs.size();i++)
    delete Childs[i];
}

bool XmlLoadColor(TiXmlNode *pChild, vec4& Color){
  std::ostringstream ErrorMessage;
  TiXmlElement *pElement;
  TiXmlAttribute *pAttrib;
  bool br=false,bg=false,bb=false,ba=false;
  if(pChild->Type() != TiXmlNode::ELEMENT){
    ErrorMessage << "Node '" << pChild->Value() << "' ist kein gültiges Element für die Farbeinlesung";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  pElement = pChild->ToElement();
  for(pAttrib = pElement->FirstAttribute();pAttrib != 0; pAttrib = pAttrib->Next()){
    if(strcmp("r",pAttrib->Name())==0){
      Color.x = atof(pAttrib->Value());
      br = true;
    }
    else if(strcmp("g",pAttrib->Name())==0){
      Color.y = atof(pAttrib->Value());
      bg = true;
    }
    else if(strcmp("b",pAttrib->Name())==0){
      Color.z = atof(pAttrib->Value());
      bb = true;
    }
    else if(strcmp("a",pAttrib->Name())==0){
      Color.w = atof(pAttrib->Value());
      ba = true;
    }
    else{
      ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' in einem " << pChild->Value() << "-Element";
      ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
      return false;
    }
  }

  if(!br){
    ErrorMessage << "Rot (r) Attribut für Farbeinlesung ein einem " << pChild->Value() << "-Tag ist nicht vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  if(!bg){
    ErrorMessage << "Grün (g) Attribut für Farbeinlesung ein einem " << pChild->Value() << "-Tag ist nicht vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  if(!bb){
    ErrorMessage << "Blau (b) Attribut für Farbeinlesung ein einem " << pChild->Value() << "-Tag ist nicht vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  if(!ba){
    ErrorMessage << "Alpha (a) Attribut für Farbeinlesung ein einem " << pChild->Value() << "-Tag ist nicht vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }

  return true;
}

bool XmlLoadRect(TiXmlNode *pChild,ThRect& Rect){
  std::ostringstream ErrorMessage;
  TiXmlElement *pElement;
  TiXmlAttribute *pAttrib;

  if(pChild->Type() != TiXmlNode::ELEMENT){
    ErrorMessage << "Node '" << pChild->Value() << "' ist kein gültiges Element für die Rechteckeinlesung";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  pElement = pChild->ToElement();

  bool bx=false,by=false,bwidth=false,bheight=false;
  for(pAttrib = pElement->FirstAttribute(); pAttrib != 0; pAttrib = pAttrib->Next()){
    if(strcmp("x",pAttrib->Name())==0){
      Rect.x = atoi(pAttrib->Value());
      bx = true;
    }
    else if(strcmp("y",pAttrib->Name())==0){
      Rect.y = atoi(pAttrib->Value());
      by = true;
    }
    else if(strcmp("width",pAttrib->Name())==0){
      Rect.width = atoi(pAttrib->Value());
      bwidth = true;
    }
    else if(strcmp("height",pAttrib->Name())==0){
      Rect.height = atoi(pAttrib->Value());
      bheight = true;
    }
    else {
      ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines " << pChild->Value() << "-Tags";
      ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
      return false;
    }
  }

  if(!bx){
    ErrorMessage << "x-Attribut innerhalb eines " << pChild->Value() << "-Tags nicht gefunden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  if(!by){
    ErrorMessage << "y-Attribut innerhalb eines " << pChild->Value() << "-Tags nicht gefunden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  if(!bwidth){
     ErrorMessage << "width-Attribut innerhalb eines " << pChild->Value() << "-Tags nicht gefunden!";
     ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
     return false;
  }
  if(!bheight){
    ErrorMessage << "height-Attribut innerhalb eines " << pChild->Value() << "-Tags nicht gefunden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  return true;
}

bool Gui::LoadStyles(const char *filename){
  FILE *datei;
  TiXmlDocument *XmlFile;
  std::ostringstream ErrorMessage;

  datei = fopen(filename,"r");
  if(datei == NULL){
    ErrorMessage << "Konnte Style Datei '" << filename << "' nicht öffnen";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }
  else
    fclose(datei);

  XmlFile = new TiXmlDocument(filename);
  bool bLoadOk = XmlFile->LoadFile();
  if(!bLoadOk)
    return false;

  TiXmlNode* pChild;
  TiXmlNode* pChild2;
  TiXmlNode* pChild3;
  TiXmlElement *pElement;
  TiXmlAttribute *pAttrib;

  ThWindowStyle TempWindowStyle;
  ThButtonStyle TempButtonStyle;
  ThElementStyle TempElementStyle;
  ThSliderStyle TempSliderStyle;
  ThRectStyle TempRectStyle;
  ThRect *TempRect;

  pChild = XmlFile->FirstChild()->NextSibling();

  if(pChild->Type() != TiXmlNode::ELEMENT){
    ThErrorMessage("GUI Error","Ungültige Style Datei\n Styles-Tag nicht am Begin der Datei gefunden");
    return false;
  }
  else if(strcmp(pChild->Value(),"Styles")!=0){
    ErrorMessage << "Falscher Tag am Begin der Datei gefunden '" << pChild->Value() << "'";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return false;
  }

  for(pChild = pChild->FirstChild(); pChild != 0; pChild = pChild->NextSibling()){
    if(pChild->Type() == TiXmlNode::ELEMENT){
      //Window Style
      if(strcmp(pChild->Value(),"WindowStyle")==0){
        bool blo=false,blu=false,blm=false,bro=false,bru=false,brm=false,br=false,bl=false,bo=false,bu=false,bfill=false,bbar=false,bcaption=false;
        for(pChild2 = pChild->FirstChild();pChild2 != 0; pChild2 = pChild2->NextSibling()){
          if(pChild2->Type() == TiXmlNode::ELEMENT){
            TempRect = NULL;
            pElement = pChild2->ToElement();
            if(strcmp("LeftTop",pChild2->Value())==0){
              TempRect = &TempWindowStyle.lo;
              blo = true;
            }
            else if(strcmp("LeftBottom",pChild2->Value())==0){
              TempRect = &TempWindowStyle.lu;
              blu = true;
            }
            else if(strcmp("LeftCenter",pChild2->Value())==0){
              TempRect = &TempWindowStyle.lm;
              blm = true;
            }
            else if(strcmp("RightTop",pChild2->Value())==0){
              TempRect = &TempWindowStyle.ro;
              bro = true;
            }
            else if(strcmp("RightBottom",pChild2->Value())==0){
              TempRect = &TempWindowStyle.ru;
              bru = true;
            }
            else if(strcmp("RightCenter",pChild2->Value())==0){
              TempRect = &TempWindowStyle.rm;
              brm = true;
            }
            else if(strcmp("Left",pChild2->Value())==0){
              TempRect = &TempWindowStyle.l;
              bl = true;
            }
            else if(strcmp("Right",pChild2->Value())==0){
              TempRect = &TempWindowStyle.r;
              br = true;
            }
            else if(strcmp("Bottom",pChild2->Value())==0){
              TempRect = &TempWindowStyle.u;
              bu = true;
            }
            else if(strcmp("Top",pChild2->Value())==0){
              TempRect = &TempWindowStyle.o;
              bo = true;
            }
            else if(strcmp("Bar",pChild2->Value())==0){
              TempRect = &TempWindowStyle.bar;
              bbar = true;
            }
            else if(strcmp("Fill",pChild2->Value())==0){
              TempRect = &TempWindowStyle.fill;
              bfill = true;
            }
            else if(strcmp("Caption",pChild2->Value())==0){
              bcaption=true;
              bool bPosition=false,bColor=false,bFont=false;
              for(pChild3 = pChild2->FirstChild(); pChild3 != 0; pChild3 = pChild3->NextSibling()){
                if(pChild3->Type() == TiXmlNode::ELEMENT){
                  if(strcmp("Color",pChild3->Value())==0){
                    bColor = true;
                    if(!XmlLoadColor(pChild3,TempWindowStyle.CaptionColor))
                      return false;
                  }
                  else if(strcmp("Position",pChild3->Value())==0){
                    bPosition = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("x",pAttrib->Name())==0)
                        TempWindowStyle.CaptionX = atoi(pAttrib->Value());
                      else if(strcmp("y",pAttrib->Name())==0)
                        TempWindowStyle.CaptionY = atoi(pAttrib->Value());
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Position-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else if(strcmp("Font",pChild3->Value())==0){
                    bFont = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("name",pAttrib->Name())==0){
                        int FontId = GetFont(pAttrib->Value());
                        if(FontId < 0){
                          ErrorMessage << "Font '" << pAttrib->Value() << "' wurde nicht definiert!";
                          ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                          return false;
                        }
                        TempWindowStyle.CaptionFont = FontId;
                      }
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Font-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else{
                    ErrorMessage << "Unbekanntes Element '" << pChild3->Value() << "' innerhalb eines Caption-Tags";
                    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                    return false;
                  }
                }
              }

              if(!bPosition){
                ThErrorMessage("GUI Error","Position-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bColor){
                ThErrorMessage("GUI Error","Color-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bFont){
                ThErrorMessage("GUI Error","Font-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
            }
            else {
              ErrorMessage << "Unbekanntes Element '" << pChild2->Value() << "' in einem WindowStyle-Tag";
              ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
              return false;
            }

            if(TempRect != NULL){
              if(!XmlLoadRect(pChild2,*TempRect))
                return false;
              if(!ComputeTexCoords(*TempRect))
                return false;
            }

          }
        }

        //Alles da?
        if(!blo){
          ThErrorMessage("GUI Error","LeftTop-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!blu){
          ThErrorMessage("GUI Error","LeftBottom-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!blm){
          ThErrorMessage("GUI Error","LeftCenter-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bro){
          ThErrorMessage("GUI Error","RightTop-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bru){
          ThErrorMessage("GUI Error","RightBottom-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!brm){
          ThErrorMessage("GUI Error","RightCenter-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!br){
          ThErrorMessage("GUI Error","Right-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bl){
          ThErrorMessage("GUI Error","Left-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bo){
          ThErrorMessage("GUI Error","Top-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bu){
          ThErrorMessage("GUI Error","Bottom-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bfill){
          ThErrorMessage("GUI Error","Fill-Element in einem WindowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bcaption){
          ThErrorMessage("GUI Error","Caption-Element in einem WIndowStyle-Tag ist nicht vorhanden!");
          return false;
        }
        WindowStyles.push_back(TempWindowStyle);
      }
      //Button Style
      else if(strcmp(pChild->Value(),"ButtonStyle")==0){
        bool blo=false,blu=false,bro=false,bru=false,br=false,bl=false,bo=false,bu=false,bfill=false,bcaption=false,bcolor=false;
        for(pChild2 = pChild->FirstChild();pChild2 != 0; pChild2 = pChild2->NextSibling()){
          if(pChild2->Type() == TiXmlNode::ELEMENT){
            TempRect = NULL;
            pElement = pChild2->ToElement();
            if(strcmp("LeftTop",pChild2->Value())==0){
              TempRect = &TempButtonStyle.lo;
              blo = true;
            }
            else if(strcmp("LeftBottom",pChild2->Value())==0){
              TempRect = &TempButtonStyle.lu;
              blu = true;
            }
            else if(strcmp("RightTop",pChild2->Value())==0){
              TempRect = &TempButtonStyle.ro;
              bro = true;
            }
            else if(strcmp("RightBottom",pChild2->Value())==0){
              TempRect = &TempButtonStyle.ru;
              bru = true;
            }
            else if(strcmp("Left",pChild2->Value())==0){
              TempRect = &TempButtonStyle.l;
              bl = true;
            }
            else if(strcmp("Right",pChild2->Value())==0){
              TempRect = &TempButtonStyle.r;
              br = true;
            }
            else if(strcmp("Bottom",pChild2->Value())==0){
              TempRect = &TempButtonStyle.u;
              bu = true;
            }
            else if(strcmp("Top",pChild2->Value())==0){
              TempRect = &TempButtonStyle.o;
              bo = true;
            }
            else if(strcmp("Fill",pChild2->Value())==0){
              TempRect = &TempButtonStyle.fill;
              bfill = true;
            }
            else if(strcmp("Caption",pChild2->Value())==0){
              bcaption=true;
              bool bColor=false,bFont=false;
              for(pChild3 = pChild2->FirstChild(); pChild3 != 0; pChild3 = pChild3->NextSibling()){
                if(pChild3->Type() == TiXmlNode::ELEMENT){
                  if(strcmp("Color",pChild3->Value())==0){
                    bColor = true;
                    if(!XmlLoadColor(pChild3,TempButtonStyle.CaptionColor))
                      return false;
                  }
                  else if(strcmp("Font",pChild3->Value())==0){
                    bFont = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("name",pAttrib->Name())==0){
                        int FontId = GetFont(pAttrib->Value());
                        if(FontId < 0){
                          ErrorMessage << "Font '" << pAttrib->Value() << "' wurde nicht definiert!";
                          ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                          return false;
                        }
                        TempButtonStyle.CaptionFont = FontId;
                      }
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Font-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else{
                    ErrorMessage << "Unbekanntes Element '" << pChild3->Value() << "' innerhalb eines Caption-Tags";
                    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                    return false;
                  }
                }
              }

              if(!bColor){
                ThErrorMessage("GUI Error","Color-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bFont){
                ThErrorMessage("GUI Error","Font-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
            }
            else if(strcmp("Color",pChild2->Value())==0){
              bcolor=true;
              if(!XmlLoadColor(pChild2,TempButtonStyle.Color))
                return false;
            }
            else {
              ErrorMessage << "Unbekanntes Element '" << pChild2->Value() << "' in einem ButtonStyle-Tag";
              ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
              return false;
            }

            if(TempRect != NULL){
              if(!XmlLoadRect(pChild2,*TempRect))
                return false;
              if(!ComputeTexCoords(*TempRect))
                return false;
            }

          }
        }

        //Alles da?
        if(!blo){
          ThErrorMessage("GUI Error","LeftTop-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!blu){
          ThErrorMessage("GUI Error","LeftBottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bro){
          ThErrorMessage("GUI Error","RightTop-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bru){
          ThErrorMessage("GUI Error","RightBottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!br){
          ThErrorMessage("GUI Error","Right-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bl){
          ThErrorMessage("GUI Error","Left-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bo){
          ThErrorMessage("GUI Error","Top-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bu){
          ThErrorMessage("GUI Error","Bottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bfill){
          ThErrorMessage("GUI Error","Fill-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bcolor){
          ThErrorMessage("GUI Error","Color-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bcaption){
          ThErrorMessage("GUI Error","Caption-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        ButtonStyles.push_back(TempButtonStyle);
      }
      //Fonts
      else if(strcmp(pChild->Value(),"Font")==0){
        bool bName=false,bFile=false,bSize=false;
        std::string Name,File;
        int Size;
        pElement = pChild->ToElement();
        for(pAttrib = pElement->FirstAttribute();pAttrib != 0; pAttrib = pAttrib->Next()){
          if(strcmp("name",pAttrib->Name())==0){
            bName = true;
            Name = pAttrib->Value();
          }
          else if(strcmp("file",pAttrib->Name())==0){
            bFile = true;
            File = "gfx/";
            File += pAttrib->Value();
          }
          else if(strcmp("size",pAttrib->Name())==0){
            bSize = true;
            Size = atoi(pAttrib->Value());
          }
          else {
            ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Font-Tags";
            ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
            return false;
          }
        }

        if(!bName){
          ThErrorMessage("GUI Error","name-Attribut innerhalb eines Font-Tags nicht vorhanden!");
          return false;
        }
        if(!bFile){
          ThErrorMessage("GUI Error","file-Attribut innerhalb eines Font-Tags nicht vorhanden!");
          return false;
        }
        if(!bSize){
          ThErrorMessage("GUI Error","size-Attribut innerhalb eines Font-Tags nicht vorhanden!");
          return false;
        }

        if(LoadFont(File.c_str(),Size,Name.c_str()) < 0)
          return false;
      }
      //Element Style
      else if(strcmp("ElementStyle",pChild->Value())==0){
        bool bcolor=false,bfill=false;
        for(pChild2 = pChild->FirstChild();pChild2 != 0; pChild2 = pChild2->NextSibling()){
          if(pChild2->Type() == TiXmlNode::ELEMENT){
            if(strcmp("Fill",pChild2->Value())==0){
              bfill = true;
              if(!XmlLoadRect(pChild2,TempElementStyle.fill))
                return false;
              if(!ComputeTexCoords(TempElementStyle.fill))
                return false;
            }
            else if(strcmp("Color",pChild2->Value())==0){
              bcolor = true;
              if(!XmlLoadColor(pChild2,TempElementStyle.Color))
                return false;
            }
            else {
              ErrorMessage << "Unbekanntes Element '" << pChild2->Value() << "' innerhalb eines " << pChild->Value() << "-Elements";
              ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
              return false;
            }
          }
        }

        if(!bcolor){
          ThErrorMessage("GUI Error","Color Element innerhalb eines ElementStyle-Tags nicht vorhanden!");
          return false;
        }

        if(!bfill){
          ThErrorMessage("GUI Error","Fill Element innerhalb eines ElementStyle-Tags nicht vorhanden!");
          return false;
        }
        ElementStyles.push_back(TempElementStyle);
      }
      //Slider Style
      else if(strcmp("SliderStyle",pChild->Value())==0){
        bool btop=false,bcenter=false,bbottom=false,bcolor=false,bcaption=false;
        for(pChild2 = pChild->FirstChild(); pChild2 != 0; pChild2 = pChild2->NextSibling()){
          if(pChild2->Type() == TiXmlNode::ELEMENT){
            if(strcmp("Top",pChild2->Value())==0){
              btop = true;
              if(!XmlLoadRect(pChild2,TempSliderStyle.top))
                return false;
              if(!ComputeTexCoords(TempSliderStyle.top))
                return false;
            }
            else if(strcmp("Center",pChild2->Value())==0){
              bcenter = true;
              if(!XmlLoadRect(pChild2,TempSliderStyle.center))
                return false;
              if(!ComputeTexCoords(TempSliderStyle.center))
                return false;
            }
            else if(strcmp("Bottom",pChild2->Value())==0){
              bbottom = true;
              if(!XmlLoadRect(pChild2,TempSliderStyle.bottom))
                return false;
              if(!ComputeTexCoords(TempSliderStyle.bottom))
                return false;
            }
            else if(strcmp("Color",pChild2->Value())==0){
              bcolor = true;
              if(!XmlLoadColor(pChild2,TempSliderStyle.Color))
                return false;
            }
            else if(strcmp("Caption",pChild2->Value())==0){
              bcaption=true;
              bool bColor=false,bFont=false;
              for(pChild3 = pChild2->FirstChild(); pChild3 != 0; pChild3 = pChild3->NextSibling()){
                if(pChild3->Type() == TiXmlNode::ELEMENT){
                  if(strcmp("Color",pChild3->Value())==0){
                    bColor = true;
                    if(!XmlLoadColor(pChild3,TempSliderStyle.CaptionColor))
                      return false;
                  }
                  else if(strcmp("Font",pChild3->Value())==0){
                    bFont = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("name",pAttrib->Name())==0){
                        int FontId = GetFont(pAttrib->Value());
                        if(FontId < 0){
                          ErrorMessage << "Font '" << pAttrib->Value() << "' wurde nicht definiert!";
                          ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                          return false;
                        }
                        TempSliderStyle.CaptionFont = FontId;
                      }
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Font-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else{
                    ErrorMessage << "Unbekanntes Element '" << pChild3->Value() << "' innerhalb eines Caption-Tags";
                    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                    return false;
                  }
                }
              }

              if(!bColor){
                ThErrorMessage("GUI Error","Color-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bFont){
                ThErrorMessage("GUI Error","Font-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
            }
            else {
              ErrorMessage << "Unbekanntes Element '" << pChild2->Value() << "' innerhalb eines SliderStyle-Tags";
              ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
              return false;
            }
          }
        }

        if(!btop){
          ThErrorMessage("GUI Error","Top Element innerhalb eines SliderStyle-Tags nicht vorhanden!");
          return false;
        }
        if(!bcenter){
          ThErrorMessage("GUI Error","Center Element innerhalb eines SliderStyle-Tags nicht vorhanden!");
          return false;
        }
        if(!bbottom){
          ThErrorMessage("GUI Error","Bottom Element innerhalb eines SliderStyle-Tags nicht vorhanden!");
          return false;
        }
        if(!bcolor){
          ThErrorMessage("GUI Error","Color Element innerhalb eines SliderStyle-Tags nicht vorhanden!");
          return false;
        }
        if(!bcaption){
          ThErrorMessage("GUI Error","Caption Element innerhalb eines SliderStyle-Tags nicht vorhanden!");
          return false;
        }
        SliderStyles.push_back(TempSliderStyle);
      }
      //Rect Style
      else if(strcmp(pChild->Value(),"RectStyle")==0){
        bool blo=false,blu=false,bro=false,bru=false,br=false,bl=false,bo=false,bu=false,bfill=false,bcaption=false,bcolor=false;
        for(pChild2 = pChild->FirstChild();pChild2 != 0; pChild2 = pChild2->NextSibling()){
          if(pChild2->Type() == TiXmlNode::ELEMENT){
            TempRect = NULL;
            pElement = pChild2->ToElement();
            if(strcmp("LeftTop",pChild2->Value())==0){
              TempRect = &TempRectStyle.lo;
              blo = true;
            }
            else if(strcmp("LeftBottom",pChild2->Value())==0){
              TempRect = &TempRectStyle.lu;
              blu = true;
            }
            else if(strcmp("RightTop",pChild2->Value())==0){
              TempRect = &TempRectStyle.ro;
              bro = true;
            }
            else if(strcmp("RightBottom",pChild2->Value())==0){
              TempRect = &TempRectStyle.ru;
              bru = true;
            }
            else if(strcmp("Left",pChild2->Value())==0){
              TempRect = &TempRectStyle.l;
              bl = true;
            }
            else if(strcmp("Right",pChild2->Value())==0){
              TempRect = &TempRectStyle.r;
              br = true;
            }
            else if(strcmp("Bottom",pChild2->Value())==0){
              TempRect = &TempRectStyle.u;
              bu = true;
            }
            else if(strcmp("Top",pChild2->Value())==0){
              TempRect = &TempRectStyle.o;
              bo = true;
            }
            else if(strcmp("Fill",pChild2->Value())==0){
              TempRect = &TempRectStyle.fill;
              bfill = true;
            }
            else if(strcmp("Caption",pChild2->Value())==0){
              bcaption=true;
              bool bColor=false,bFont=false,bPosition=false;
              for(pChild3 = pChild2->FirstChild(); pChild3 != 0; pChild3 = pChild3->NextSibling()){
                if(pChild3->Type() == TiXmlNode::ELEMENT){
                  if(strcmp("Color",pChild3->Value())==0){
                    bColor = true;
                    if(!XmlLoadColor(pChild3,TempRectStyle.CaptionColor))
                      return false;
                  }
                  else if(strcmp("Font",pChild3->Value())==0){
                    bFont = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("name",pAttrib->Name())==0){
                        int FontId = GetFont(pAttrib->Value());
                        if(FontId < 0){
                          ErrorMessage << "Font '" << pAttrib->Value() << "' wurde nicht definiert!";
                          ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                          return false;
                        }
                        TempRectStyle.CaptionFont = FontId;
                      }
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Font-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else if(strcmp("Position",pChild3->Value())==0){
                    bPosition = true;
                    pElement = pChild3->ToElement();
                    for(pAttrib = pElement->FirstAttribute();pAttrib != 0;pAttrib = pAttrib->Next()){
                      if(strcmp("x",pAttrib->Name())==0){
                        TempRectStyle.CaptionX = atoi(pAttrib->Value());
                      }
                      else if(strcmp("y",pAttrib->Name())==0){
                        TempRectStyle.CaptionY = atoi(pAttrib->Value());
                      }
                      else {
                        ErrorMessage << "Unbekanntes Attribut '" << pAttrib->Name() << "' innerhalb eines Position-Tags";
                        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                        return false;
                      }
                    }
                  }
                  else{
                    ErrorMessage << "Unbekanntes Element '" << pChild3->Value() << "' innerhalb eines Caption-Tags";
                    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
                    return false;
                  }
                }
              }

              if(!bColor){
                ThErrorMessage("GUI Error","Color-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bFont){
                ThErrorMessage("GUI Error","Font-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
              if(!bPosition){
                ThErrorMessage("GUI Error","Position-Element in einem Caption-Tag nicht vorhanden!");
                return false;
              }
            }
            else if(strcmp("Color",pChild2->Value())==0){
              bcolor=true;
              if(!XmlLoadColor(pChild2,TempRectStyle.Color))
                return false;
            }
            else {
              ErrorMessage << "Unbekanntes Element '" << pChild2->Value() << "' in einem ButtonStyle-Tag";
              ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
              return false;
            }

            if(TempRect != NULL){
              if(!XmlLoadRect(pChild2,*TempRect))
                return false;
              if(!ComputeTexCoords(*TempRect))
                return false;
            }

          }
        }

        //Alles da?
        if(!blo){
          ThErrorMessage("GUI Error","LeftTop-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!blu){
          ThErrorMessage("GUI Error","LeftBottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bro){
          ThErrorMessage("GUI Error","RightTop-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bru){
          ThErrorMessage("GUI Error","RightBottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!br){
          ThErrorMessage("GUI Error","Right-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bl){
          ThErrorMessage("GUI Error","Left-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bo){
          ThErrorMessage("GUI Error","Top-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bu){
          ThErrorMessage("GUI Error","Bottom-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bfill){
          ThErrorMessage("GUI Error","Fill-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bcolor){
          ThErrorMessage("GUI Error","Color-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        if(!bcaption){
          ThErrorMessage("GUI Error","Caption-Element in einem ButtonStyle-Tag ist nicht vorhanden!");
          return false;
        }
        RectStyles.push_back(TempRectStyle);
      }
      else {
        ErrorMessage << "Unbekanntes Element '" << pChild->Value() << "' innerhalb des Styles-Tag";
        ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
        return false;
      }
    }
  }
  return true;
}

bool Gui::ComputeTexCoords(ThRect& Rect){
  if(GuiTexture == NULL){
    ThErrorMessage("GUI Error","Fehler: Noch keine GUI Textur geladen");
    return false;
  }

  float fStepX = 1.0f/(float)GuiTexture->GetWidth();
  float fStepY = 1.0f/(float)GuiTexture->GetHeight();
  Rect.fx1 = (float)Rect.x * fStepX + fStepX/2.0f;
  Rect.fy1 = 1.0f - ((float)Rect.y * fStepY + fStepY/2.0f);
  Rect.fx2 = (float)(Rect.x + Rect.width - 1) * fStepX + fStepX/2.0f;
  Rect.fy2 = 1.0f - ((float)(Rect.y + Rect.height - 1) * fStepY + fStepY/2.0f);

  return true;
}

bool Gui::LoadGuiTexture(const char *filename){
  if(GuiTexture != NULL)
    delete GuiTexture;
  GuiTexture = new Texture2D();
  return GuiTexture->LoadTGA(filename,TH_NEAREST | TH_CLAMP_S | TH_CLAMP_T);
}

bool Gui::LoadGuiShader(const char *filename){
  if(GuiShader != NULL)
    delete GuiShader;
  GuiShader = new XMLShader();
  return GuiShader->LoadFromFile(filename);
}

void Gui::DrawRect(ThRect Rect, vec4 Color){
  vec4 pos;
  pos.z = 0.5f;
  pos.w = 1.0f;

  //Vertex 1
  pos.x = (float)Rect.x;
  pos.y = (float)Rect.y;
  vbuffer->AddVertexData(pos.f,3);
  vbuffer->AddVertexData(Color);
  vbuffer->AddVertexData(Rect.fx1);
  vbuffer->AddVertexData(Rect.fy1);

  //Vertex 2
  pos.x = (float)(Rect.x+Rect.width);
  pos.y = (float)Rect.y;
  vbuffer->AddVertexData(pos.f,3);
  vbuffer->AddVertexData(Color);
  vbuffer->AddVertexData(Rect.fx2);
  vbuffer->AddVertexData(Rect.fy1);

  //Vertex 3
  pos.x = (float)(Rect.x+Rect.width);
  pos.y = (float)(Rect.y+Rect.height);
  vbuffer->AddVertexData(pos.f,3);
  vbuffer->AddVertexData(Color);
  vbuffer->AddVertexData(Rect.fx2);
  vbuffer->AddVertexData(Rect.fy2);

  //Vertex 4
  pos.x = (float)Rect.x;
  pos.y = (float)(Rect.y+Rect.height);
  vbuffer->AddVertexData(pos.f,3);
  vbuffer->AddVertexData(Color);
  vbuffer->AddVertexData(Rect.fx1);
  vbuffer->AddVertexData(Rect.fy2);
  iRectCount++;
}

void Gui::DrawFont(int set, int x, int y, vec4 Color, const char *Text){
  ThGuiFont TempFont;
  TempFont.num = set;
  TempFont.x = (float)x;
  TempFont.y = (float)y;
  TempFont.Color = Color;
  TempFont.Text = Text;
  iTextCount++;
  FontDrawCalls.push_back(TempFont);
}

void Gui::FinishElement(){
  ThGuiElement TempElement;
  TempElement.iTextCount = iTextCount;
  TempElement.iRectCount = iRectCount;
  TempElement.iOrder = 0;
  GuiElements.push_back(TempElement);
}

ThWindowStyle* Gui::GetWindowStyle(int num){
  if(num < 0 || num >= WindowStyles.size())
    return NULL;
  return &WindowStyles[num];
}

ThButtonStyle* Gui::GetButtonStyle(int num){
  if(num < 0 || num >= ButtonStyles.size())
    return NULL;
  return &ButtonStyles[num];
}

ThElementStyle* Gui::GetElementStyle(int num){
  if(num < 0 || num >= ElementStyles.size())
    return NULL;
  return &ElementStyles[num];
}

ThSliderStyle* Gui::GetSliderStyle(int num){
  if(num < 0 || num >= SliderStyles.size())
    return NULL;
  return &SliderStyles[num];
}

ThRectStyle* Gui::GetRectStyle(int num){
  if(num < 0 || num >= RectStyles.size())
    return NULL;
  return &RectStyles[num];
}

GuiWindow* Gui::CreateWindow(GuiEventRect* Father,int x, int y, int width, int height, bool Fixed, int Style, const char *Caption, void* Creator){
  std::ostringstream ErrorMessage;
  GuiWindow* TempWindow = new GuiWindow(this,Fixed,Style,Caption);
  TempWindow->SetRect(x,y,width,height);
  TempWindow->SetCreator(Creator);
  if(GetWindowStyle(Style)==NULL){
    ErrorMessage << "Windowstyle mit der Nummer '" << Style << "' nicht vorhanden. Nur " << WindowStyles.size() << " Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    delete TempWindow;
    return NULL;
  }
  if(Father == NULL)
    AddChild(TempWindow);
  else
    Father->AddChild(TempWindow);
  return TempWindow;
}

GuiButton* Gui::CreateButton(class GuiEventRect* Father,int x, int y, int width, int height, int NormalStyle, int HoverStyle, const char *Caption, void* Creator){
  std::ostringstream ErrorMessage;
  GuiButton* TempButton = new GuiButton(this,NormalStyle,HoverStyle,Caption);
  TempButton->SetRect(x,y,width,height);
  TempButton->SetCreator(Creator);
  if(GetButtonStyle(NormalStyle)==NULL || GetButtonStyle(HoverStyle)==NULL){
    ErrorMessage << "Buttonstyle mit der Nummer '" << NormalStyle << "' oder '" << HoverStyle << "' nicht vorhanden. Nur " << ButtonStyles.size() << " Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    delete TempButton;
    return NULL;
  }
  if(Father == NULL)
    AddChild(TempButton);
  else
    Father->AddChild(TempButton);
  return TempButton;
}

GuiSliderVertical* Gui::CreateSliderVertical(class GuiEventRect* Father, int x, int y, int Height, float Min, float Max, int Style, int ChildStyle, int ChildHoverStyle, const char *Caption, void* Creator){
  std::ostringstream ErrorMessage;
  if(GetElementStyle(ChildStyle)==NULL || GetElementStyle(ChildHoverStyle)==NULL){
    ErrorMessage << "Elementstyle mit der Nummer '" << ChildStyle << "' oder '" << ChildHoverStyle << "' nicht vorhanden. Nur " << ElementStyles.size() << "Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  if(GetSliderStyle(Style)==NULL){
    ErrorMessage << "Sliderstyle mit der Nummer '" << Style << "' nicht vorhanden. Nur " << SliderStyles.size() << " Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  GuiSliderVertical* TempSlider = new GuiSliderVertical(this,Height,Min,Max,Style,ChildStyle,ChildHoverStyle,Caption);
  TempSlider->SetPosition(x,y);
  TempSlider->SetCreator(Creator);
  if(Father == NULL)
    AddChild(TempSlider);
  else
    Father->AddChild(TempSlider);
  return TempSlider;
}

GuiColorView* Gui::CreateColorView(GuiEventRect* Father, int x, int y, int Style, void* Creator){
  std::ostringstream ErrorMessage;
  if(GetElementStyle(Style)==NULL){
    ErrorMessage << "Elementstyle mit der Nummer '" << Style << "' nicht vorhanden. Nur " << SliderStyles.size() << " Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  GuiColorView* TempColorView = new GuiColorView(this,Style);
  TempColorView->SetPosition(x,y);
  TempColorView->SetCreator(Creator);
  if(Father == NULL)
    AddChild(TempColorView);
  else
    Father->AddChild(TempColorView);
  return TempColorView;
}

GuiSelect* Gui::CreateSelect(GuiEventRect* Father, int x, int y, ThGuiSelectInit InitStruct, void* Creator){
  std::ostringstream ErrorMessage;
  if(GetElementStyle(InitStruct.ButtonUpStyle) == NULL || GetElementStyle(InitStruct.ButtonUpHoverStyle)==NULL){
    ErrorMessage << "Select Up-Botton Style Nr. '" << InitStruct.ButtonUpStyle << "' oder '" << InitStruct.ButtonUpHoverStyle << "' nicht definiert!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  if(GetElementStyle(InitStruct.ButtonDownStyle) == NULL || GetElementStyle(InitStruct.ButtonDownHoverStyle)==NULL){
    ErrorMessage << "Select Down-Botton Style Nr. '" << InitStruct.ButtonUpStyle << "' oder '" << InitStruct.ButtonUpHoverStyle << "' nicht definiert!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  if(GetRectStyle(InitStruct.Style) == NULL){
    ErrorMessage << "Select Style Nr. '" << InitStruct.Style << "' nicht definiert!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  if(GetRectStyle(InitStruct.SelectStyle) == NULL || GetRectStyle(InitStruct.SelectHoverStyle) == NULL || GetRectStyle(InitStruct.SelectSelectedStyle) == NULL){
    ErrorMessage << "Select Element Style Nr. '" << InitStruct.SelectStyle << "','" << InitStruct.SelectHoverStyle << "' oder '" << InitStruct.SelectSelectedStyle << "' nicht definiert!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    return NULL;
  }
  GuiSelect* TempSelect = new GuiSelect(this,InitStruct);
  TempSelect->SetPosition(x,y);
  TempSelect->SetCreator(Creator);
  if(Father == NULL)
    AddChild(TempSelect);
  else
    Father->AddChild(TempSelect);
  return TempSelect;
}

GuiTextInput* Gui::CreateTextInput(class GuiEventRect* Father, int x, int y, int width, int height, int NormalStyle, int ActiveStyle, int MaxLength, const char *Caption, void* Creator){
  std::ostringstream ErrorMessage;
  GuiTextInput* TempTextInput = new GuiTextInput(this,NormalStyle,ActiveStyle,MaxLength,Caption);
  TempTextInput->SetRect(x,y,width,height);
  TempTextInput->SetCreator(Creator);
  if(GetRectStyle(NormalStyle)==NULL || GetRectStyle(ActiveStyle)==NULL){
    ErrorMessage << "Rectstyle mit der Nummer '" << NormalStyle << "' oder '" << ActiveStyle << "' nicht vorhanden. Nur " << RectStyles.size() << " Styles vorhanden!";
    ThErrorMessage("GUI Error",ErrorMessage.str().c_str());
    delete TempTextInput;
    return NULL;
  }
  if(Father == NULL)
    AddChild(TempTextInput);
  else
    Father->AddChild(TempTextInput);
  return TempTextInput;
}

GuiText* Gui::CreateText(GuiEventRect* Father, int x, int y, int Font, const char *text, void* Creator){
  std::ostringstream ErrorMessage;
  GuiText *TempText = new GuiText(this,Font,text);
  TempText->SetPosition(x,y);
  TempText->SetCreator(Creator);
  if(Father == NULL)
    AddChild(TempText);
  else
    Father->AddChild(TempText);
  return TempText;
}

void Gui::AddChild(GuiEventRect *Child){
  Childs.push_back(Child);
}

void Gui::Draw(){
  GuiShader->GetShader()->Use();

  /*glBegin(GL_QUADS);
    glTexCoord2f(0.0f,1.0f); glVertex2f(0.0f,0.0f);
    glTexCoord2f(1.0f,1.0f); glVertex2f(200.0f,0.0f);
    glTexCoord2f(1.0f,0.0f); glVertex2f(200.0f,200.0f);
    glTexCoord2f(0.0f,0.0f); glVertex2f(0.0f, 200.0f);
  glEnd();*/

  vbuffer->ResetLocalData();
  FontDrawCalls.resize(0);
  GuiElements.resize(0);
  for(int i=Childs.size()-1;i>=0;i--){
    iTextCount = 0;
    iRectCount = 0;
    Childs[i]->Draw(0,0);
    FinishElement();
  }

  vbuffer->UploadData();
  int current=0,CurrentText=0;
  for(int i=0;i<GuiElements.size();i++){
    if(GuiElements[i].iRectCount > 0){
      GuiTexture->BindToShader(0);
      vbuffer->DrawRange(TH_VB_COLOR | TH_VB_TEXCOORD0,current,GuiElements[i].iRectCount * 4);
      current += GuiElements[i].iRectCount * 4;
    }
    for(int j=0;j<GuiElements[i].iTextCount;j++){
      glPushMatrix();
      glLoadIdentity();
      glTranslatef(FontDrawCalls[j+CurrentText].x,FontDrawCalls[j+CurrentText].y,0.0f);
      glColor4fv(FontDrawCalls[j+CurrentText].Color.f);
      glPrint(FontDrawCalls[j+CurrentText].num,FontDrawCalls[j+CurrentText].Text.c_str());
      glPopMatrix();
    }
    CurrentText += GuiElements[i].iTextCount;
  }
  UnloadShader();
}

void Gui::MouseDown(int button, int x, int y){
  for(int i=0;i<Childs.size();i++){
    if(Childs[i]->MouseDown(NULL,button,x,y)){
      if(i!=0){
        GuiEventRect *Temp = Childs[i];
        for(int j=i;j>0;j--){
          Childs[j] = Childs[j-1];
        }
        Childs[0] = Temp;
      }
      break;
    }
  }
}

void Gui::MouseUp(int button, int x, int y){
  for(int i=0;i<Childs.size();i++){
    if(Childs[i]->MouseUp(NULL,button,x,y))
      break;
  }
}

void Gui::MouseMove(int xabs, int yabs, int xrel, int yrel){
  for(int i=0;i<Childs.size();i++){
    if(Childs[i]->MouseMove(NULL,xabs,yabs,xrel,yrel)){
      for(int j=0;j<Childs.size();j++){
        if(i==j)
          continue;
        Childs[j]->MouseOut();
      }
      break;
    }
  }
}

void Gui::TextIn(char text){
  if(TextReciever != NULL)
    TextReciever->TextIn(text);
}


//GuiEventRect
GuiEventRect::GuiEventRect(Gui* pGui){
  ix=0;
  iy=0;
  iwidth=0;
  iheight=0;
  active=true;
  bMouseIn = false;
  bFocus = false;
  theGui = pGui;
}

GuiEventRect::~GuiEventRect(){
  for(int i=0;i<Childs.size();i++)
    delete Childs[i];
}

bool GuiEventRect::MouseDown(class GuiEventRect *Father,int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  x-=ix;
  y-=iy;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseDown(this,button,x,y);
  return true;
}

bool GuiEventRect::MouseUp(class GuiEventRect *Father,int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  x-=ix;
  y-=iy;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseUp(this,button,x,y);
  return true;
}

bool GuiEventRect::MouseMove(class GuiEventRect *Father, int xabs, int yabs,int xrel, int yrel){
  if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  xabs-=ix;
  yabs-=iy;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseMove(this,xabs,yabs,xrel,yrel);
  return true;
}

void GuiEventRect::TextIn(char text){
  //Nothing to do
}

void GuiEventRect::MouseOut(){
  bMouseIn = false;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseOut();
}

void GuiEventRect::LooseFocus(){
  bFocus = false;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->LooseFocus();
}

void GuiEventRect::AddChild(class GuiEventRect *Child){
  Childs.push_back(Child);
}


//Gui Window
GuiWindow::GuiWindow(Gui* pGui,bool fixed, int Style, const char *pCaption) : GuiEventRect(pGui){
  bFixed = fixed;
  iStyle = Style;
  bDraging = false;
  if(pCaption != NULL)
    Caption = pCaption;
}

bool GuiWindow::MouseDown(class GuiEventRect *Father,int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    if(bFocus)
      LooseFocus();
    return false;
  }
  bMouseIn = true;
  bFocus = true;
  x-=ix;
  y-=iy;
  if(y < theGui->GetWindowStyle(iStyle)->o.height + theGui->GetWindowStyle(iStyle)->bar.height && !Caption.empty() && button==0)
    bDraging = true;
  else{
    bool bFound = false;
    if(!Caption.empty()){
      ThWindowStyle *WindowStyle = theGui->GetWindowStyle(iStyle);
      y -= WindowStyle->o.height + WindowStyle->bar.height;
    }
    for(int i=0;i<Childs.size();i++)
      if(Childs[i]->MouseDown(this,button,x,y)){
        bFound = true;
        for(int j=0;j<Childs.size();j++){
          if(i==j)
            continue;
          Childs[j]->LooseFocus();
        }
      }
    if(!bFound)
      for(int i=0;i<Childs.size();i++)
        Childs[i]->LooseFocus();
  }
  return true;
}

bool GuiWindow::MouseUp(class GuiEventRect *Father,int button, int x, int y){
  if(bDraging)
    bDraging = false;
  else {
    if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
    x-=ix;
    y-=iy;
    if(!Caption.empty()){
      ThWindowStyle *WindowStyle = theGui->GetWindowStyle(iStyle);
      y -= WindowStyle->o.height + WindowStyle->bar.height;
    }
    for(int i=0;i<Childs.size();i++)
      Childs[i]->MouseUp(this,button,x,y);
  }
  return true;
}

bool GuiWindow::MouseMove(class GuiEventRect *Father, int xabs, int yabs,int xrel, int yrel){
  if(bDraging){
    ix += xrel;
    iy += yrel;
    if(ix < 0) ix = 0; if(ix > iWidth - iwidth) ix = iWidth - iwidth;
    if(iy < 0) iy = 0; if(iy > iHeight - iheight) iy = iHeight - iheight;
  }
  else{
    if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight || !active){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
    xabs-=ix;
    yabs-=iy;
    if(!Caption.empty()){
      ThWindowStyle *WindowStyle = theGui->GetWindowStyle(iStyle);
      yabs -= WindowStyle->o.height + WindowStyle->bar.height;
    }
    for(int i=0;i<Childs.size();i++)
      Childs[i]->MouseMove(this,xabs,yabs,xrel,yrel);
  }
  return true;
}

void GuiWindow::MouseOut(){
  bDraging = false;
  bMouseIn = false;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseOut();
}

void GuiWindow::Draw(int x, int y){
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  vec4 TempColor;
  TempColor.set(1.0f);
  ThWindowStyle *WindowStyle = theGui->GetWindowStyle(iStyle);
  if(WindowStyle != NULL){
    //Links Oben
    TempRect = WindowStyle->lo;
    TempRect.x = x;
    TempRect.y = y;
    theGui->DrawRect(TempRect,TempColor);

    //Rechts Oben
    TempRect = WindowStyle->ro;
    TempRect.x = x + iwidth - TempRect.width;
    TempRect.y = y;
    theGui->DrawRect(TempRect,TempColor);

    //Oben
    TempRect = WindowStyle->o;
    TempRect.x = x + WindowStyle->lo.width;
    TempRect.y = y;
    TempRect.width = iwidth - WindowStyle->lo.width - WindowStyle->ro.width;
    theGui->DrawRect(TempRect,TempColor);

    //Links Unten
    TempRect = WindowStyle->lu;
    TempRect.x = x;
    TempRect.y = y + iheight - TempRect.height;
    theGui->DrawRect(TempRect,TempColor);

    //Rechts Unten
    TempRect = WindowStyle->ru;
    TempRect.x = x + iwidth - TempRect.width;
    TempRect.y = y + iheight - TempRect.height;
    theGui->DrawRect(TempRect,TempColor);

    //Unten
    TempRect = WindowStyle->u;
    TempRect.x = x + WindowStyle->lu.width;
    TempRect.y = y + iheight - TempRect.height;
    TempRect.width = iwidth - WindowStyle->lu.width - WindowStyle->ru.width;
    theGui->DrawRect(TempRect,TempColor);

    if(Caption.empty()){
      //Links
      TempRect = WindowStyle->l;
      TempRect.x = x;
      TempRect.y = y + WindowStyle->lo.height;
      TempRect.height = iheight - WindowStyle->lo.height - WindowStyle->lu.height;
      theGui->DrawRect(TempRect,TempColor);

      //Rechts
      TempRect = WindowStyle->r;
      TempRect.x = x + iwidth - TempRect.width;
      TempRect.y = y + WindowStyle->ro.height;
      TempRect.height = iheight - WindowStyle->ro.height - WindowStyle->ru.height;
      theGui->DrawRect(TempRect,TempColor);

      //Fill
      TempRect = WindowStyle->fill;
      TempRect.x = x + WindowStyle->l.width;
      TempRect.y = y + WindowStyle->o.height;
      TempRect.height = iheight - WindowStyle->o.height - WindowStyle->u.height;
      TempRect.width = iwidth - WindowStyle->l.width - WindowStyle->r.width;
      theGui->DrawRect(TempRect,TempColor);
    }
    else{
      //Links
      TempRect = WindowStyle->l;
      TempRect.x = x;
      TempRect.y = y + WindowStyle->lo.height + WindowStyle->lm.height;
      TempRect.height = iheight - WindowStyle->lo.height - WindowStyle->lu.height - WindowStyle->lm.height;
      theGui->DrawRect(TempRect,TempColor);

      //Rechts
      TempRect = WindowStyle->r;
      TempRect.x = x + iwidth - TempRect.width;
      TempRect.y = y + WindowStyle->ro.height + WindowStyle->rm.height;
      TempRect.height = iheight - WindowStyle->ro.height - WindowStyle->ru.height - WindowStyle->rm.height;
      theGui->DrawRect(TempRect,TempColor);

      //Fill
      TempRect = WindowStyle->fill;
      TempRect.x = x + WindowStyle->l.width;
      TempRect.y = y + WindowStyle->o.height + WindowStyle->bar.height;
      TempRect.height = iheight - WindowStyle->o.height - WindowStyle->u.height - WindowStyle->bar.height;
      TempRect.width = iwidth - WindowStyle->l.width - WindowStyle->r.width;
      theGui->DrawRect(TempRect,TempColor);

      //Links Mitte
      TempRect = WindowStyle->lm;
      TempRect.x = x;
      TempRect.y = y + WindowStyle->lo.height;
      theGui->DrawRect(TempRect,TempColor);

      //Rechts Mitte
      TempRect = WindowStyle->rm;
      TempRect.x = x + iwidth - TempRect.width;
      TempRect.y = y + WindowStyle->ro.height;
      theGui->DrawRect(TempRect,TempColor);

      //Bar
      TempRect = WindowStyle->bar;
      TempRect.x = x + WindowStyle->lm.width;
      TempRect.y = y + WindowStyle->o.height;
      TempRect.width = iwidth - WindowStyle->lm.width - WindowStyle->rm.width;
      theGui->DrawRect(TempRect,TempColor);

      //Caption
      theGui->DrawFont(WindowStyle->CaptionFont,x+WindowStyle->CaptionX,y+WindowStyle->CaptionY,WindowStyle->CaptionColor,Caption.c_str());

      y += WindowStyle->o.height + WindowStyle->bar.height;
    }
  }

  for(int i=0;i<Childs.size();i++)
    Childs[i]->Draw(x,y);
}

//GuiButton
GuiButton::GuiButton(Gui* pGui,int NormalStyle, int HoverStyle, const char* pCaption) : GuiEventRect(pGui){
  iNormalStyle = NormalStyle;
  iHoverStyle = HoverStyle;
  Caption = pCaption;
  OnPress = NULL;
}

bool GuiButton::MouseDown(class GuiEventRect *Father, int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active)
    return false;
  if(button == 0 && OnPress != NULL)
    OnPress(Creator);
  return true;
}

bool GuiButton::MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel){
  if(!active)
    return false;
  if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  return true;
}

void GuiButton::MouseOut(){
  bMouseIn = false;
}

void GuiButton::Draw(int x, int y){
  if(!active)
    return;
  int iTextWidth,iTextHeight;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThButtonStyle* ButtonStyle = (bMouseIn) ? theGui->GetButtonStyle(iHoverStyle) : theGui->GetButtonStyle(iNormalStyle);

  //Links Oben
  TempRect = ButtonStyle->lo;
  TempRect.x = x;
  TempRect.y = y;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts Oben
  TempRect = ButtonStyle->ro;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Oben
  TempRect = ButtonStyle->o;
  TempRect.x = x + ButtonStyle->lo.width;
  TempRect.y = y;
  TempRect.width = iwidth - ButtonStyle->lo.width - ButtonStyle->ro.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Links Unten
  TempRect = ButtonStyle->lu;
  TempRect.x = x;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts Unten
  TempRect = ButtonStyle->ru;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Unten
  TempRect = ButtonStyle->u;
  TempRect.x = x + ButtonStyle->lu.width;
  TempRect.y = y + iheight - TempRect.height;
  TempRect.width = iwidth - ButtonStyle->lu.width - ButtonStyle->ru.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Links
  TempRect = ButtonStyle->l;
  TempRect.x = x;
  TempRect.y = y + ButtonStyle->lo.height;
  TempRect.height = iheight - ButtonStyle->lo.height - ButtonStyle->lu.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts
  TempRect = ButtonStyle->r;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + ButtonStyle->ro.height;
  TempRect.height = iheight - ButtonStyle->ro.height - ButtonStyle->ru.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Fill
  TempRect = ButtonStyle->fill;
  TempRect.x = x + ButtonStyle->l.width;
  TempRect.y = y + ButtonStyle->o.height;
  TempRect.height = iheight - ButtonStyle->o.height - ButtonStyle->u.height;
  TempRect.width = iwidth - ButtonStyle->l.width - ButtonStyle->r.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Caption
  GetTextSize(iTextWidth,iTextHeight,ButtonStyle->CaptionFont,Caption.c_str());
  if(iTextWidth != -1 && iTextHeight != -1){
    theGui->DrawFont(ButtonStyle->CaptionFont,x + (iwidth - iTextWidth) / 2, y + (iheight - iTextHeight) / 2,ButtonStyle->CaptionColor,Caption.c_str());
  }
}

//GuiTextInput
GuiTextInput::GuiTextInput(Gui* pGui,int NormalStyle, int ActiveStyle, int MaxLength, const char *pCaption) : GuiEventRect(pGui) {
  iNormalStyle = NormalStyle;
  iActiveStyle = ActiveStyle;
  iMaxLength = MaxLength;
  theGui = pGui;
  Caption = pCaption;
  OnChange = NULL;
  OnLooseFocus = NULL;
}

bool GuiTextInput::MouseDown(class GuiEventRect *Father, int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active)
    return false;
  if(button == 0){
    bFocus = true;
    theGui->SetTextReciever(this);
  }
  return true;
}

void GuiTextInput::LooseFocus(){
  if(bFocus){
    bFocus = false;
    if(this == theGui->GetTextReciever())
      theGui->SetTextReciever(NULL);
    if(OnLooseFocus != NULL)
      OnLooseFocus(Creator);
  }
}

void GuiTextInput::TextIn(char text){
  if(text == 8 && !Caption.empty()){
    Caption.erase(Caption.end()-1);
    if(OnChange != NULL)
      OnChange(Creator);
  }
  if(text == '\n' && OnLooseFocus != NULL)
    OnLooseFocus(Creator);
  if(Caption.length() < iMaxLength && text >= 32 && text <= 126){
    Caption.insert(Caption.end(),text);
    if(OnChange != NULL)
      OnChange(Creator);
  }
}

void GuiTextInput::SetCaption(const char *pCaption){
  Caption = pCaption;
}

void GuiTextInput::Draw(int x, int y){
  if(!active)
    return;
  int iTextWidth,iTextHeight;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThRectStyle* ButtonStyle = (bFocus) ? theGui->GetRectStyle(iActiveStyle) : theGui->GetRectStyle(iNormalStyle);

  //Links Oben
  TempRect = ButtonStyle->lo;
  TempRect.x = x;
  TempRect.y = y;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts Oben
  TempRect = ButtonStyle->ro;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Oben
  TempRect = ButtonStyle->o;
  TempRect.x = x + ButtonStyle->lo.width;
  TempRect.y = y;
  TempRect.width = iwidth - ButtonStyle->lo.width - ButtonStyle->ro.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Links Unten
  TempRect = ButtonStyle->lu;
  TempRect.x = x;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts Unten
  TempRect = ButtonStyle->ru;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Unten
  TempRect = ButtonStyle->u;
  TempRect.x = x + ButtonStyle->lu.width;
  TempRect.y = y + iheight - TempRect.height;
  TempRect.width = iwidth - ButtonStyle->lu.width - ButtonStyle->ru.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Links
  TempRect = ButtonStyle->l;
  TempRect.x = x;
  TempRect.y = y + ButtonStyle->lo.height;
  TempRect.height = iheight - ButtonStyle->lo.height - ButtonStyle->lu.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Rechts
  TempRect = ButtonStyle->r;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + ButtonStyle->ro.height;
  TempRect.height = iheight - ButtonStyle->ro.height - ButtonStyle->ru.height;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Fill
  TempRect = ButtonStyle->fill;
  TempRect.x = x + ButtonStyle->l.width;
  TempRect.y = y + ButtonStyle->o.height;
  TempRect.height = iheight - ButtonStyle->o.height - ButtonStyle->u.height;
  TempRect.width = iwidth - ButtonStyle->l.width - ButtonStyle->r.width;
  theGui->DrawRect(TempRect,ButtonStyle->Color);

  //Caption
  if(!Caption.empty())
    theGui->DrawFont(ButtonStyle->CaptionFont, x + ButtonStyle->CaptionX, y + ButtonStyle->CaptionY ,ButtonStyle->CaptionColor,Caption.c_str());
}

//GuiActionElement
GuiActionElement::GuiActionElement(Gui *pGui, int NormalStyle, int HoverStyle) : GuiEventRect(pGui){
  iNormalStyle = NormalStyle;
  iHoverStyle = HoverStyle;
  ThElementStyle *ElementStyle = theGui->GetElementStyle(NormalStyle);
  iwidth = ElementStyle->fill.width;
  iheight = ElementStyle->fill.height;
}

bool GuiActionElement::MouseMove(class GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel){
  if(!active)
    return false;
  if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  return true;
}

void GuiActionElement::MouseOut(){
  bMouseIn = false;
}

void GuiActionElement::Draw(int x, int y){
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThElementStyle* ElementStyle = (bMouseIn) ? theGui->GetElementStyle(iHoverStyle) : theGui->GetElementStyle(iNormalStyle);

  //Fill
  TempRect = ElementStyle->fill;
  TempRect.x = x;
  TempRect.y = y;
  theGui->DrawRect(TempRect,ElementStyle->Color);
}

//GuiSliderVertical
GuiSliderVertical::GuiSliderVertical(Gui *pGui, int Height, float Min, float Max, int Style, int ChildStyle, int ChildHoverStyle, const char *pCaption) : GuiEventRect(pGui){
  GuiSliderVerticalChild* Child;
  ThSliderStyle *SliderStyle = theGui->GetSliderStyle(Style);
  ThElementStyle *ElementStyle = theGui->GetElementStyle(ChildStyle);

  fMin = Min;
  fMax = Max;
  fValue = Max;
  iStyle = Style;
  iSliderHeight = Height;
  Caption = pCaption;
  OnChange = NULL;
  iChildStyle = ChildStyle;

  GetTextSize(iTextWidth,iTextHeight,SliderStyle->CaptionFont,pCaption);
  if(iTextHeight < 0) iTextHeight = 0;
  if(iTextWidth < 0) iTextWidth = 0;

  iwidth = (iTextWidth > ElementStyle->fill.width) ? iTextWidth : ElementStyle->fill.width + 4;
  iheight = iTextHeight + Height + ElementStyle->fill.height + 3;
  iTextHeight += ElementStyle->fill.height / 2 + 3;

  Child = new GuiSliderVerticalChild(pGui,ChildStyle,ChildHoverStyle,(iwidth - ElementStyle->fill.width) / 2, iTextHeight - ElementStyle->fill.height / 2,iTextHeight - ElementStyle->fill.height / 2,iTextHeight + iSliderHeight - ElementStyle->fill.height / 2);
  AddChild(Child);
}

bool GuiSliderVertical::MouseMove(GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel){
  if(Father == Childs[0]){
    fValue = (1.0f - (float)yrel / (float)iSliderHeight) * (fMax - fMin) + fMin;
    if(OnChange != NULL)
      OnChange(Creator);
  }
  else{
    if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight || !active){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
    xabs-=ix;
    yabs-=iy;
    Childs[0]->MouseMove(this,xabs,yabs,xrel,yrel);
  }
  return true;
}

void GuiSliderVertical::SetValue(float value){
  ThElementStyle *ElementStyle = theGui->GetElementStyle(iChildStyle);
  fValue = value;
  if(fValue > fMax) fValue = fMax;
  if(fValue < fMin) fValue = fMin;
  Childs[0]->SetPosition((iwidth - ElementStyle->fill.width) / 2, iTextHeight - ElementStyle->fill.height / 2 + (int)((float)iSliderHeight * (1.0f - ((fValue - fMin) / (fMax - fMin)))));
}

void GuiSliderVertical::Draw(int x, int y){
  int versatz;
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThSliderStyle* SliderStyle = theGui->GetSliderStyle(iStyle);

  versatz = (iwidth - SliderStyle->top.width) / 2;

  //Top
  TempRect = SliderStyle->top;
  TempRect.x = x + versatz;
  TempRect.y = y + iTextHeight;
  theGui->DrawRect(TempRect,SliderStyle->Color);

  //Center
  TempRect = SliderStyle->center;
  TempRect.x = x + versatz;
  TempRect.y = y + iTextHeight + SliderStyle->top.height;
  TempRect.height = iSliderHeight - SliderStyle->top.height - SliderStyle->bottom.height;
  theGui->DrawRect(TempRect,SliderStyle->Color);

  //Bottom
  TempRect = SliderStyle->bottom;
  TempRect.x = x + versatz;
  TempRect.y = y + iTextHeight + iSliderHeight - SliderStyle->bottom.height;
  theGui->DrawRect(TempRect,SliderStyle->Color);

  //Caption
  if(!Caption.empty()){
    theGui->DrawFont(SliderStyle->CaptionFont,x + (iwidth - iTextWidth) / 2,y,SliderStyle->CaptionColor,Caption.c_str());
  }

  Childs[0]->Draw(x,y);
}

//GuiSliderVerticalChild
GuiSliderVerticalChild::GuiSliderVerticalChild(Gui* pGui, int NormalStyle, int HoverStyle, int x, int y, int Min, int Max) : GuiActionElement(pGui,NormalStyle,HoverStyle){
  iMin = Min;
  iMax = Max;
  ix = x;
  iy = y;
  bDraging = false;
}

bool GuiSliderVerticalChild::MouseDown(GuiEventRect *Father, int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  if(button == 0)
    bDraging = true;
  return true;
}

bool GuiSliderVerticalChild::MouseUp(GuiEventRect *Father, int button, int x, int y){
  if(button == 0 && bDraging)
    bDraging = false;
  return true;
}

bool GuiSliderVerticalChild::MouseMove(GuiEventRect *Father, int xabs, int yabs, int xrel, int yrel){
  if(bDraging){
    /*if((xabs < ix || xabs > ix + iwidth) && bMouseIn)
      MouseOut();
    else{*/
      iy += yrel;
      if(iy > iMax) iy = iMax;
      if(iy < iMin) iy = iMin;
      Father->MouseMove(this,xabs,yabs,0,iy - iMin);
    //}
  }
  else {
    if(!active)
      return false;
    if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
  }
  return true;
}

void GuiSliderVerticalChild::MouseOut(){
  bMouseIn = false;
  bDraging = false;
}

//GuiColorView
GuiColorView::GuiColorView(Gui *pGui,int Style) : GuiEventRect(pGui){
  ThElementStyle* ElementStyle = pGui->GetElementStyle(Style);
  iStyle = Style;
  Color = ElementStyle->Color;
  iwidth = ElementStyle->fill.width;
  iheight = ElementStyle->fill.height;
}

void GuiColorView::Draw(int x, int y){
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThElementStyle* ElementStyle = theGui->GetElementStyle(iStyle);

  //Fill
  TempRect = ElementStyle->fill;
  TempRect.x = x;
  TempRect.y = y;
  theGui->DrawRect(TempRect,Color);
}

//GuiSelectButton
GuiSelectButton::GuiSelectButton(Gui* pGui, int NormalStyle, int HoverStyle) : GuiActionElement(pGui,NormalStyle,HoverStyle){
  //nix
}

bool GuiSelectButton::MouseDown(GuiEventRect* Father, int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  if(button==0)
    Father->MouseDown(this,button,x,y);
  return true;
}

bool GuiSelectButton::MouseUp(GuiEventRect* Father, int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  if(button==0)
    Father->MouseUp(this,button,x,y);
  return true;
}

//GuiSelectElement
GuiSelectElement::GuiSelectElement(Gui *pGui, int NormalStyle, int HoverStyle, int SelectedStyle) : GuiEventRect(pGui){
  iNormalStyle = NormalStyle;
  iHoverStyle = HoverStyle;
  iSelectedStyle = SelectedStyle;
  bSelected = false;
}


bool GuiSelectElement::MouseDown(GuiEventRect* Father, int button, int x, int y){
  if(Father == NULL)
    bSelected = true;
  else {
    if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
    if(button==0)
      Father->MouseDown(this,button,x,y);
  }
  return true;
}

bool GuiSelectElement::MouseUp(GuiEventRect* Father, int button, int x, int y){
  if(Father == NULL)
    bSelected = false;
  else {
    if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
      if(bMouseIn)
        MouseOut();
      return false;
    }
    bMouseIn = true;
    if(button==0)
      Father->MouseUp(this,button,x,y);
  }
  return true;
}

void GuiSelectElement::Draw(int x, int y){
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;
  ThRectStyle* RectStyle;
  if(!bSelected)
    RectStyle = (bMouseIn) ? theGui->GetRectStyle(iHoverStyle) : theGui->GetRectStyle(iNormalStyle);
  else
    RectStyle = theGui->GetRectStyle(iSelectedStyle);

  //Links Oben
  TempRect = RectStyle->lo;
  TempRect.x = x;
  TempRect.y = y;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Rechts Oben
  TempRect = RectStyle->ro;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Oben
  TempRect = RectStyle->o;
  TempRect.x = x + RectStyle->lo.width;
  TempRect.y = y;
  TempRect.width = iwidth - RectStyle->lo.width - RectStyle->ro.width;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Links Unten
  TempRect = RectStyle->lu;
  TempRect.x = x;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Rechts Unten
  TempRect = RectStyle->ru;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + iheight - TempRect.height;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Unten
  TempRect = RectStyle->u;
  TempRect.x = x + RectStyle->lu.width;
  TempRect.y = y + iheight - TempRect.height;
  TempRect.width = iwidth - RectStyle->lu.width - RectStyle->ru.width;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Links
  TempRect = RectStyle->l;
  TempRect.x = x;
  TempRect.y = y + RectStyle->lo.height;
  TempRect.height = iheight - RectStyle->lo.height - RectStyle->lu.height;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Rechts
  TempRect = RectStyle->r;
  TempRect.x = x + iwidth - TempRect.width;
  TempRect.y = y + RectStyle->ro.height;
  TempRect.height = iheight - RectStyle->ro.height - RectStyle->ru.height;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Fill
  TempRect = RectStyle->fill;
  TempRect.x = x + RectStyle->l.width;
  TempRect.y = y + RectStyle->o.height;
  TempRect.height = iheight - RectStyle->o.height - RectStyle->u.height;
  TempRect.width = iwidth - RectStyle->l.width - RectStyle->r.width;
  theGui->DrawRect(TempRect,RectStyle->Color);

  //Caption (fehlt noch)
  if(!Caption.empty())
    theGui->DrawFont(RectStyle->CaptionFont,x + RectStyle->CaptionX,y + RectStyle->CaptionY,RectStyle->CaptionColor,Caption.c_str());
}

//GuiSelect
GuiSelect::GuiSelect(Gui* pGui,ThGuiSelectInit InitStruct) : GuiEventRect(pGui){
  ThElementStyle* ButtonUpStyle = pGui->GetElementStyle(InitStruct.ButtonUpStyle);
  ThElementStyle* ButtonDownStyle = pGui->GetElementStyle(InitStruct.ButtonDownStyle);
  ThRectStyle* SelectStyle = pGui->GetRectStyle(InitStruct.SelectStyle);
  ThRectStyle* Style = pGui->GetRectStyle(InitStruct.Style);

  //Init Values
  OnChange = NULL;
  iSelected = -1;
  iShowFrom = 0;

  //Copy Values
  iwidth = InitStruct.Width;
  iStyle = InitStruct.Style;
  iSelectStyle = InitStruct.SelectStyle;
  iSelectHoverStyle = InitStruct.SelectHoverStyle;
  iSelectSelectedStyle = InitStruct.SelectSelectedStyle;
  iShowAnz = InitStruct.MaxElementsVisible;
  iSpace = InitStruct.Space;

  //Compute Values
  iElementWidth = iwidth - Style->l.width - Style->r.width - ButtonUpStyle->fill.width - 1;
  iheight = Style->o.height + Style->u.height + iShowAnz * (SelectStyle->o.height + SelectStyle->u.height + GetMaxFontHeight(SelectStyle->CaptionFont) + iSpace);

  //Create Childs
  Up = new GuiSelectButton(pGui,InitStruct.ButtonUpStyle,InitStruct.ButtonUpHoverStyle);
  Up->SetPosition(Style->l.width + iElementWidth + 1,Style->o.height);
  Down = new GuiSelectButton(pGui,InitStruct.ButtonDownStyle,InitStruct.ButtonDownHoverStyle);
  Down->SetPosition(Style->l.width + iElementWidth + 1,iheight - Style->u.height - ButtonDownStyle->fill.height);
  //Scrollbar fehlt
  ScrollBar = NULL;
}

GuiSelect::~GuiSelect(){
  if(Up != NULL)
    delete Up;
  if(Down != NULL)
    delete Down;
  if(ScrollBar != NULL)
    delete ScrollBar;
}

void GuiSelect::Update(){
  ThRectStyle *Style = theGui->GetRectStyle(iStyle);
  ThRectStyle *SelectStyle = theGui->GetRectStyle(iSelectStyle);
  int SelectHeight = SelectStyle->o.height + SelectStyle->u.height + GetMaxFontHeight(SelectStyle->CaptionFont) + iSpace;
  for(int i=0;i<Childs.size();i++)
    Childs[i]->SetActive(false);

  for(int i=iShowFrom;i<iShowFrom+iShowAnz;i++){
    if(i >= Childs.size())
      break;
    Childs[i]->SetActive(true);
    Childs[i]->SetPosition(Style->l.width,SelectHeight * (i-iShowFrom)+Style->o.height);
  }
}

bool GuiSelect::MouseDown(GuiEventRect *Father, int button, int x, int y){
  if(!active)
    return false;
  if(Father == Up){
     iShowFrom--;
     if(iShowFrom < 0) iShowFrom = 0;
     Update();
  }
  else if(Father == Down){
    iShowFrom++;
    if(iShowFrom > Childs.size() - 1) iShowFrom = Childs.size() - 1;
    Update();
  }
  else {
    int i;
    for(i=0;i<Childs.size();i++)
      if(Father == Childs[i])
        break;
    if(i < Childs.size()){
      if(i != iSelected){
        if(iSelected >= 0 && iSelected < Childs.size())
          Childs[iSelected]->MouseUp(NULL,button,x,y);
        Childs[i]->MouseDown(NULL,button,x,y);
        iSelected = i;
        if(OnChange != NULL)
          OnChange(Creator);
      }
    }
    else{
      if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight){
        if(bMouseIn)
          MouseOut();
        return false;
      }
      bMouseIn = true;
      x-=ix;
      y-=iy;
      Up->MouseDown(this,button,x,y);
      Down->MouseDown(this,button,x,y);
      for(int i=0;i<Childs.size();i++)
        Childs[i]->MouseDown(this,button,x,y);
    }
  }
  return true;
}

bool GuiSelect::MouseUp(class GuiEventRect *Father,int button, int x, int y){
  if(x < ix || x > ix+iwidth || y < iy || y > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  x-=ix;
  y-=iy;
  Up->MouseUp(this,button,x,y);
  Down->MouseUp(this,button,x,y);
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseUp(this,button,x,y);
  return true;
}

bool GuiSelect::MouseMove(class GuiEventRect *Father, int xabs, int yabs,int xrel, int yrel){
  if(xabs < ix || xabs > ix+iwidth || yabs < iy || yabs > iy + iheight || !active){
    if(bMouseIn)
      MouseOut();
    return false;
  }
  bMouseIn = true;
  xabs-=ix;
  yabs-=iy;
  Up->MouseMove(this,xabs,yabs,xrel,yrel);
  Down->MouseMove(this,xabs,yabs,xrel,yrel);
  for(int i=0;i<Childs.size();i++)
    Childs[i]->MouseMove(this,xabs,yabs,xrel,yrel);
  return true;
}

void GuiSelect::Draw(int x, int y){
  if(!active)
    return;
  x+=ix;
  y+=iy;
  ThRect TempRect;;
  ThRectStyle *RectStyle = theGui->GetRectStyle(iStyle);

    //Links Oben
    TempRect = RectStyle->lo;
    TempRect.x = x;
    TempRect.y = y;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Rechts Oben
    TempRect = RectStyle->ro;
    TempRect.x = x + iwidth - TempRect.width;
    TempRect.y = y;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Oben
    TempRect = RectStyle->o;
    TempRect.x = x + RectStyle->lo.width;
    TempRect.y = y;
    TempRect.width = iwidth - RectStyle->lo.width - RectStyle->ro.width;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Links Unten
    TempRect = RectStyle->lu;
    TempRect.x = x;
    TempRect.y = y + iheight - TempRect.height;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Rechts Unten
    TempRect = RectStyle->ru;
    TempRect.x = x + iwidth - TempRect.width;
    TempRect.y = y + iheight - TempRect.height;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Unten
    TempRect = RectStyle->u;
    TempRect.x = x + RectStyle->lu.width;
    TempRect.y = y + iheight - TempRect.height;
    TempRect.width = iwidth - RectStyle->lu.width - RectStyle->ru.width;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Links
    TempRect = RectStyle->l;
    TempRect.x = x;
    TempRect.y = y + RectStyle->lo.height;
    TempRect.height = iheight - RectStyle->lo.height - RectStyle->lu.height;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Rechts
    TempRect = RectStyle->r;
    TempRect.x = x + iwidth - TempRect.width;
    TempRect.y = y + RectStyle->ro.height;
    TempRect.height = iheight - RectStyle->ro.height - RectStyle->ru.height;
    theGui->DrawRect(TempRect,RectStyle->Color);

    //Fill
    TempRect = RectStyle->fill;
    TempRect.x = x + RectStyle->l.width;
    TempRect.y = y + RectStyle->o.height;
    TempRect.height = iheight - RectStyle->o.height - RectStyle->u.height;
    TempRect.width = iwidth - RectStyle->l.width - RectStyle->r.width;
    theGui->DrawRect(TempRect,RectStyle->Color);

  Up->Draw(x,y);
  Down->Draw(x,y);
  for(int i=0;i<Childs.size();i++)
    Childs[i]->Draw(x,y);
}

void GuiSelect::AddOption(const char* Caption){
  ThRectStyle* Style = theGui->GetRectStyle(iStyle);
  ThRectStyle* ElementStyle = theGui->GetRectStyle(iSelectStyle);
  GuiSelectElement *TempElement = new GuiSelectElement(theGui,iSelectStyle,iSelectHoverStyle,iSelectSelectedStyle);
  TempElement->SetRect(Style->l.width,Style->o.height,iElementWidth,ElementStyle->o.height + ElementStyle->u.height + GetMaxFontHeight(ElementStyle->CaptionFont));
  TempElement->SetCaption(Caption);
  Childs.push_back(TempElement);
  Update();
}

void GuiSelect::InsertOption(int insert_pos, const char* Caption){
  ThRectStyle* Style = theGui->GetRectStyle(iStyle);
  ThRectStyle* ElementStyle = theGui->GetRectStyle(iSelectStyle);
  GuiSelectElement *TempElement = new GuiSelectElement(theGui,iSelectStyle,iSelectHoverStyle,iSelectSelectedStyle);
  TempElement->SetRect(Style->l.width,Style->o.height,iElementWidth,ElementStyle->o.height + ElementStyle->u.height + GetMaxFontHeight(ElementStyle->CaptionFont));
  TempElement->SetCaption(Caption);
  Childs.insert(Childs.begin()+insert_pos,TempElement);
  Update();
}

void GuiSelect::RemoveOption(int num){
  if(num < 0 || num >= Childs.size())
    return;
  delete Childs[num];
  Childs.erase(Childs.begin()+num);
  Update();
  if(iSelected == num)
    iSelected = -1;
}

void GuiSelect::RemoveAllOptions(){
  iSelected = -1;
  for(int i=0;i<Childs.size();i++)
    delete Childs[i];
  Childs.resize(0);
  Update();
}

void GuiSelect::ChangeOption(int num,const char* Caption){
  if(num < 0 || num > Childs.size()-1)
    return;
  ((GuiSelectElement*)Childs[num])->SetCaption(Caption);
}

//GuiText
void GuiText::PrintToCaption(const char *str,...){
  char text[1000];
  va_list	ap;
  va_start(ap,str);
    vsprintf(text,str,ap);
  va_end(ap);

  Caption = text;
}
