/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#include "GLee.h"
#include "thEngine.h"
#include <stdio.h>
#include "shader.h"
#include <string>

void UnloadShader(){
  glUseProgramObjectARB(0);
}

Shader::Shader(const char *name){
  int len=strlen(name);
  cName = new char[len+1];
  strcpy(cName,name);
  linked=false;
  glhShaderProgram = glCreateProgramObjectARB();
  glhVertexShader = 0;
  glhFragmentShader = 0;
  gliUniforms = NULL;
  gliAttribs = NULL;
  gluiUniformBuffers = NULL;
  iInstanceAttribute = -1;
  iInstanceUniform = -1;
}

Shader::~Shader(){
  if(glhVertexShader != 0){
    glDetachObjectARB(glhShaderProgram,glhVertexShader);
    glDeleteObjectARB(glhVertexShader);
  }
  if(glhFragmentShader != 0){
    glDetachObjectARB(glhShaderProgram,glhFragmentShader);
    glDeleteObjectARB(glhFragmentShader);
  }
  glDeleteObjectARB(glhShaderProgram);
  delete []cName;
  if(gliUniforms != NULL)
    delete []gliUniforms;
  if(gliAttribs != NULL)
    delete []gliAttribs;
  if(gluiUniformBuffers != NULL){
    for(int i=0;i<iUniformCount;i++)
      if(gluiUniformBuffers[i] != 0)
        glDeleteBuffers(1,&gluiUniformBuffers[i]);

    delete []gluiUniformBuffers;
  }
}

bool Shader::ShaderSource(const char *source, GLhandleARB& glhShader, GLenum ShaderType){
  int iCompileStatus;
  glhShader = glCreateShaderObjectARB(ShaderType);
  if(glhShader == 0){
    ThErrorMessage("Shader Error","Couldn't Create Shader");
    return false;
  }
  glShaderSourceARB(glhShader,1,(const char**)&source,NULL);
  glCompileShaderARB(glhShader);
  glGetObjectParameterivARB(glhShader,GL_OBJECT_COMPILE_STATUS_ARB,&iCompileStatus);
  if(iCompileStatus != 1){
    getError(glhShader);
    return false;
  }
  glAttachObjectARB(glhShaderProgram,glhShader);
  return true;
}

bool Shader::LoadVertexShader(const char *lpcFileName){
  FILE *datei;
  char *content;
  char temp;
  int i=0,iCompileStatus;
  if(linked)
    return false;
  datei = fopen(lpcFileName,"r");
  if(datei == NULL)
    return false;
  for(i=0;!feof(datei);i++)
    fread(&temp,1,1,datei);
  fseek(datei,0,SEEK_SET);
  content = new char[i];
  memset(content,0,i);
  fread(content,1,i,datei);
  fclose(datei);
  if(!ShaderSource(content,glhVertexShader,GL_VERTEX_SHADER_ARB)){
    delete []content;
    return false;
  }
  delete []content;
  return true;
}

bool Shader::LoadGeometryShader(const char *lpcFileName){
  FILE *datei;
  char *content;
  char temp;
  int i=0,iCompileStatus;
  if(linked)
    return false;
  datei = fopen(lpcFileName,"r");
  if(datei == NULL)
    return false;
  for(i=0;!feof(datei);i++)
    fread(&temp,1,1,datei);
  fseek(datei,0,SEEK_SET);
  content = new char[i];
  memset(content,0,i);
  fread(content,1,i,datei);
  fclose(datei);
  if(!ShaderSource(content,glhGeometryShader,GL_GEOMETRY_SHADER_EXT)){
    delete []content;
    return false;
  }
  delete []content;
  return true;
}

bool Shader::LoadFragmentShader(const char *lpcFileName){
  FILE *datei;
  char *content;
  char temp;
  int i=0,iCompileStatus;
  if(linked)
    return false;
  datei = fopen(lpcFileName,"r");
  if(datei == NULL)
    return false;
  for(i=0;!feof(datei);i++)
    fread(&temp,1,1,datei);
  fseek(datei,0,SEEK_SET);
  content = new char[i];
  memset(content,0,i);
  fread(content,1,i,datei);
  fclose(datei);
  if(!ShaderSource(content,glhFragmentShader,GL_FRAGMENT_SHADER_ARB)){
    delete []content;
    return false;
  }
  delete []content;
  return true;
}

bool Shader::LoadShaderSource(const char *source, int num){
  switch(num){
    case TH_VERTEX_SHADER:
      if(ShaderSource(source,glhVertexShader,GL_VERTEX_SHADER_ARB)){
        CheckWarnings(glhVertexShader,"Vertex Shader Compiler Warnings");
        return true;
      }
      else
        return false;
      break;
    case TH_GEOMETRY_SHADER:
      if(ShaderSource(source,glhGeometryShader,GL_GEOMETRY_SHADER_EXT)){
        CheckWarnings(glhGeometryShader,"Geometry Shader Compiler Warnings");
        return true;
      }
      else
        return false;
      break;
    case TH_FRAGMENT_SHADER:
      if(ShaderSource(source,glhFragmentShader,GL_FRAGMENT_SHADER_ARB)){
        CheckWarnings(glhFragmentShader,"Fragment Shader Compiler Warnings");
        return true;
      }
      else
        return false;
      break;
    default:
      return false;
  }
}

void Shader::SetGSParameter(char option, int value){
  switch(option){
    case TH_GS_INPUT:
      glProgramParameteriEXT(glhShaderProgram,GL_GEOMETRY_INPUT_TYPE_EXT,value);
      break;
    case TH_GS_OUTPUT:
      glProgramParameteriEXT(glhShaderProgram,GL_GEOMETRY_OUTPUT_TYPE_EXT,value);
      break;
    case TH_GS_OUTPUT_NUM:
      glProgramParameteriEXT(glhShaderProgram,GL_GEOMETRY_VERTICES_OUT_EXT,value);
      break;
  }
  #ifdef TH_SHADER_DEBUG
  char meldung[200];
  sprintf(meldung,"GS Shader '%s' Parameter %i to %i",cName,option,value);
  ThGetGLError(meldung);
  #endif
}

bool Shader::Validate(){
  if(!linked){
    ThErrorMessage("Shader Error","Validation before linking");
    return false;
  }
  GLint isValid;
  ThErrorMessage("Shader Linker Messages","");
  getError(glhShaderProgram);
  glValidateProgramARB(glhShaderProgram);
  glGetObjectParameterivARB(glhShaderProgram,GL_OBJECT_VALIDATE_STATUS_ARB,&isValid);
  //if(isValid != 1){
    ThErrorMessage("Shader Validation:","");
    getError(glhShaderProgram);
    //return false;
  //}
  linked = true;
  return true;
}

bool Shader::Link(){
  if(linked){
    ThErrorMessage("Shader Error","Already Linked");
    return false;
  }
  GLint isLinked;
  glLinkProgramARB(glhShaderProgram);
  glGetObjectParameterivARB(glhShaderProgram,GL_OBJECT_LINK_STATUS_ARB,&isLinked);
  if(isLinked != 1){
    getError(glhShaderProgram);
    linked = false;
    return false;
  }
  if(!CheckWarnings(glhShaderProgram,cName) || !CheckWarnings(glhVertexShader,"Vertex Shader") || !CheckWarnings(glhFragmentShader,"Fragment Shader"))
    return false;
  linked = true;
  #ifdef TH_SHADER_DEBUG
    Validate();
  #endif
  return true;
}

void Shader::Use(){
  if(!linked)
    return;
  glUseProgramObjectARB(glhShaderProgram);
  for(int i=0;i<ShaderConstants.size();i++)
    ShaderConstants[i].ptr->Update(ShaderConstants[i].num,this);
}

void Shader::getError(GLhandleARB obj){
  int infologLength = 0;
  int charsWritten  = 0;
  char *infoLog;
  FILE *errorlog;
  char name[200];

  glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB,&infologLength);

  if (infologLength > 0)
  {
    infoLog = new char[infologLength+1];
    glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
    infoLog[infologLength]='\0';
    sprintf(name,"shader error :: %s",cName);
      ThErrorMessage(name,infoLog);
	delete []infoLog;
  }
}

bool Shader::CheckWarnings(GLhandleARB obj,const char *title){
  int infologLength = 0;
  int charsWritten = 0;
  char *infoLog;
  FILE *errorlog;
  std::string ErrorMessage;
  char name[200];

  glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infologLength);

  if(infologLength > 1){
    infoLog = new char[infologLength+1];
    glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
    infoLog[infologLength]='\0';
    if(strcmp("vertex shader will run in hardware. fragment shader will run in hardware.",infoLog)==0){
      delete []infoLog;
      return true;
    }
    ErrorMessage = infoLog;
    errorlog = fopen("engine.log","a");
    fprintf(errorlog,"%s :: %s:\n%s\n",cName,title,infoLog);
    fclose(errorlog);
    delete []infoLog;
    if(ErrorMessage.find("software") == std::string::npos && ErrorMessage.find("Software") == std::string::npos)
      return true;
    else
      return false;
  }
  return true;
}

GLint Shader::GetAttrib(const char* name){
  return glGetAttribLocationARB(glhShaderProgram,name);
}

GLint Shader::GetAttributeLocation(int num){
  if(gliAttribs != NULL)
    return gliAttribs[num];
  else
    return -1;
}

GLint Shader::GetUniform(const char* name){
  char meldung[200];
  GLint temp = glGetUniformLocationARB(glhShaderProgram,name);
  if(temp == -1){
    sprintf(meldung,"Shader '%s':\nCouldn't Find Uniform Var '%s' in Shader Context",cName,name);
    #ifndef TH_FLEXIBLE_SHADERS
    ThErrorMessage("Shader Error",meldung);
    #else
    ThErrorMessage("Shader Warning",meldung);
    #endif
  }
  return temp;
}

bool Shader::LoadUniforms(const char* names){
  char *name;
  int i,j,anz=1,last=0,len;
  for(i=0;names[i]!='\0';i++){
    if(names[i]=='\n')
      anz++;
  }
  gliUniforms = new GLint[anz];
  gluiUniformBuffers = new GLuint[anz];
  iUniformCount = anz;
  for(i=0;i<anz;i++){
    len=0;
    for(j=last;names[j]!='\0' && names[j]!='\n';j++);
    len = j-last;
    name = new char[len+1];
    for(j=0;j<len;j++)
      name[j] = names[last+j];
    name[len]='\0';
    gliUniforms[i]=GetUniform(name);
    #ifndef TH_FLEXIBLE_SHADERS
    if(gliUniforms[i]==-1)
      return false;
    #endif
    delete []name;
    last += j+1;
    gluiUniformBuffers[i]=0;
  }
  return true;
}

bool Shader::LoadAttributes(const char* names){
  char *name;
  int i,j,anz=1,last=0,len;
  for(i=0;names[i]!='\0';i++){
    if(names[i]=='\n')
      anz++;
  }
  gliAttribs = new GLint[anz];
  iAttribCount = anz;
  for(i=0;i<anz;i++){
    len=0;
    for(j=last;names[j]!='\0' && names[j]!='\n';j++);
    len = j-last;
    name = new char[len+1];
    for(j=0;j<len;j++)
      name[j] = names[last+j];
    name[len]='\0';
    gliAttribs[i]=GetAttrib(name);
    #ifndef TH_FLEXIBLE_SHADERS
    if(gliAttribs[i]==-1)
      return false;
    #endif
    delete []name;
    last += j+1;
  }
  return true;
}

void Shader::SetUniform(int num,float f1){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform1fARB(gliUniforms[num],f1);
}

void Shader::SetUniform(int num,float f1,float f2){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform2fARB(gliUniforms[num],f1,f2);
}

void Shader::SetUniform(int num,float f1,float f2,float f3){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform3fARB(gliUniforms[num],f1,f2,f3);
}

void Shader::SetUniform(int num,float f1,float f2,float f3,float f4){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform4fARB(gliUniforms[num],f1,f2,f3,f4);
}

void Shader::SetUniform(int num,int size1, int size2, float *ptr){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  switch(size1){
    case 1:
      glUniform1fvARB(gliUniforms[num],size2,ptr);
      break;
    case 2:
      glUniform2fvARB(gliUniforms[num],size2,ptr);
      break;
    case 3:
      glUniform3fvARB(gliUniforms[num],size2,ptr);
      break;
    case 4:
      glUniform4fvARB(gliUniforms[num],size2,ptr);
      break;
  }
}

void Shader::SetUniform(int num,vec4 vektor){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform4fvARB(gliUniforms[num],1,vektor.f);
}

void Shader::SetUniform(int num,vec3 vektor){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform3fvARB(gliUniforms[num],1,vektor.f);
}

void Shader::SetUniform(int num, int size, bool transponse,float *array){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  switch(size){
    case 2:
      glUniformMatrix2fv(gliUniforms[num],1,transponse,array);
      break;
    case 3:
      glUniformMatrix3fv(gliUniforms[num],1,transponse,array);
      break;
    case 4:
      glUniformMatrix4fv(gliUniforms[num],1,transponse,array);
      break;
  }
}

void Shader::SetUniform(int num,int i1){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform1iARB(gliUniforms[num],i1);
}

void Shader::SetUniform(int num,int i1,int i2){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform2iARB(gliUniforms[num],i1,i2);
}

void Shader::SetUniform(int num,int i1, int i2, int i3){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform3iARB(gliUniforms[num],i1,i2,i3);
}

void Shader::SetUniform(int num,int i1,int i2,int i3,int i4){
  #ifdef TH_FLEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniform4iARB(gliUniforms[num],i1,i2,i3,i4);
}

void Shader::BindBufferToUniform(int num, GLuint Buffer){
  #ifdef TH_FEXIBLE_SHADERS
  if(gliUniforms[num]==-1)
    return;
  #endif
  glUniformBufferEXT(glhShaderProgram, gliUniforms[num], Buffer);
}

bool Shader::GenBuffer(int num, int size, GLenum mode){
  if(gliUniforms[num]==-1 || gluiUniformBuffers[num]!=0)
    return false;
  glGenBuffers(1,&gluiUniformBuffers[num]);
  glBindBuffer(GL_UNIFORM_BUFFER_EXT,gluiUniformBuffers[num]);
  glBufferData(GL_UNIFORM_BUFFER_EXT,size,NULL,mode);
  glBindBuffer(GL_UNIFORM_BUFFER_EXT,0);
  glUniformBufferEXT(glhShaderProgram, gliUniforms[num], gluiUniformBuffers[num]);
  return true;
}

bool Shader::UploadData(int num, int start, int size, const void *ptr){
  if(gliUniforms[num]==-1 || gluiUniformBuffers[num]==0)
    return false;
  glBindBuffer(GL_UNIFORM_BUFFER_EXT,gluiUniformBuffers[num]);
  glBufferSubData(GL_UNIFORM_BUFFER_EXT,start,size,ptr);
  return true;
}

void Shader::AddShaderConstant(int num, ShaderConstant *ptr) {
  ShaderConstantReference temp;
  temp.num = num;
  temp.ptr = ptr;
  ShaderConstants.push_back(temp);
}

//Shader Constant Float
void ShaderConstantFloat::Set(const void *ptr){
  data = *((const float*)ptr);
}

void ShaderConstantFloat::Update(int num, Shader *ptr){
  ptr->SetUniform(num,data);
}

//Shader Constant Vec3
void ShaderConstantVec3::Set(const void *ptr){
  data = *((const vec3*)ptr);
}

void ShaderConstantVec3::Update(int num,Shader *ptr){
  ptr->SetUniform(num,data);
}

//Shader Constsant Vec4
void ShaderConstantVec4::Set(const void *ptr){
  data = *((const vec4*)ptr);
}

void ShaderConstantVec4::Update(int num,Shader *ptr){
  ptr->SetUniform(num,data);
}

//Shader Constant Mat4a
void ShaderConstantMat4a::Set(const void *ptr){
  data = *((const mat4a*)ptr);
}

void ShaderConstantMat4a::Update(int num, Shader *ptr){
  ptr->SetUniform(num,4,false,data.f);
}
