/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#include "sound.h"

Sound::Sound(){
  bDataLoaded = false;
}

void Sound::SetPitch(float value){
  if(!bDataLoaded)
    return;
  alSourcef(aluiSource,AL_PITCH,value);
}

void Sound::SetPosition(float x, float y, float z){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_POSITION,x,y,z);
}

void Sound::SetPosition(float *array){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_POSITION,array[0],array[1],array[2]);
}

void Sound::SetPosition(vec4 Position){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_POSITION,Position.x,Position.y,Position.z);
}

void Sound::SetVelocity(float x, float y, float z){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_VELOCITY,x,y,z);
}

void Sound::SetVelocity(float *array){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_VELOCITY,array[0],array[1],array[2]);
}

void Sound::SetVelocity(vec3 Velocity){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_VELOCITY,Velocity.x,Velocity.y,Velocity.z);
}

void Sound::SetDirection(float *array){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_DIRECTION,array[0],array[1],array[2]);
}

void Sound::SetDirection(vec3 Direction){
  if(!bDataLoaded)
    return;
  alSource3f(aluiSource,AL_DIRECTION,Direction.x,Direction.y,Direction.z);
}

void Sound::SetRolloff(float value){
  if(!bDataLoaded)
    return;
  alSourcef(aluiSource,AL_ROLLOFF_FACTOR,value);
}

void Sound::SetRelative(bool value){
  if(!bDataLoaded)
    return;
  alSourcei(aluiSource,AL_SOURCE_RELATIVE,value);
}

void Sound::SetRepeat(bool value){
  if(!bDataLoaded)
    return;
  alSourcei(aluiSource,AL_LOOPING,value);
}

void Sound::SetVolume(float value){
  if(!bDataLoaded)
    return;
  alSourcef(aluiSource,AL_GAIN,value);
}

void Sound::Play(){
  if(!bDataLoaded)
    return;
  if(!Playing())
    alSourcePlay(aluiSource);
}

void Sound::Stop(){
  if(!bDataLoaded)
    return;
  if(Playing())
    alSourceStop(aluiSource);
}

bool Sound::Playing(){
  if(!bDataLoaded)
    return false;
  ALenum state;
  alGetSourcei(aluiSource,AL_SOURCE_STATE,&state);
  return (state == AL_PLAYING);
}

#define BUFFER_SIZE   262144

bool SoundOgg::LoadOgg(const char *filename){
  int endian = 0;             // 0 for Little-Endian, 1 for Big-Endian
  int bitStream;
  long bytes;
  char array[BUFFER_SIZE];    // Local fixed size array
  FILE *f;

  if(bDataLoaded)
    return false;

  // Open for binary reading
  f = fopen(filename, "rb");
  if(!f){
    fclose(f);
    return false;
  }

  vorbis_info *pInfo;
  OggVorbis_File oggFile;

  if(ov_open(f, &oggFile, NULL, 0) < 0){
    fclose(f);
    return false;
  }
  // Get some information about the OGG file
  pInfo = ov_info(&oggFile, -1);

  // Check the number of channels... always use 16-bit samples
  if (pInfo->channels == 1)
    format = AL_FORMAT_MONO16;
  else
    format = AL_FORMAT_STEREO16;
  // end if

  // The frequency of the sampling rate
  freq = pInfo->rate;

  do {
    // Read up to a buffer's worth of decoded sound data
    bytes = ov_read(&oggFile, array, BUFFER_SIZE, endian, 2, 1, &bitStream);
    // Append to end of buffer
    DataBuffer.insert(DataBuffer.end(), array, array + bytes);
  } while (bytes > 0);

  ov_clear(&oggFile);

  alGenBuffers(1,&aluiBuffer1);
  alGenSources(1,&aluiSource);
  alBufferData(aluiBuffer1,format,&DataBuffer[0],DataBuffer.size(),freq);
  alSourcei(aluiSource, AL_BUFFER, aluiBuffer1);

  bDataLoaded = true;
  return true;
}

SoundOgg::~SoundOgg(){
  if(!bDataLoaded)
    return;
  alDeleteBuffers(1,&aluiBuffer1);
  alDeleteSources(1,&aluiSource);
  DataBuffer.empty();
}

bool SoundOggStream::LoadOgg(const char *filename){
  int bytes;
  FILE *f;

  if(bDataLoaded)
    return false;

  // Open for binary reading
  f = fopen(filename, "rb");
  if(!f){
    fclose(f);
    return false;
  }

  vorbis_info *pInfo;

  if(ov_open(f, &oggFile, NULL, 0) < 0){
    fclose(f);
    return false;
  }
  // Get some information about the OGG file
  pInfo = ov_info(&oggFile, -1);

  // Check the number of channels... always use 16-bit samples
  if (pInfo->channels == 1)
    format = AL_FORMAT_MONO16;
  else
    format = AL_FORMAT_STEREO16;
  // end if

  // The frequency of the sampling rate
  freq = pInfo->rate;

  alGenSources(1,&aluiSource);

  alGenBuffers(1,&aluiBuffer1);
  bytes = Stream(aluiBuffer1);
  if( bytes <= 0 ){
    alDeleteSources(1,&aluiSource);
    alDeleteBuffers(1,&aluiBuffer1);
    ov_clear(&oggFile);
    return false;
  }
  alSourceQueueBuffers(aluiSource,1,&aluiBuffer1);

  alGenBuffers(1,&aluiBuffer2);
  bytes = Stream(aluiBuffer2);
  if( bytes <= 0){
    alSourceUnqueueBuffers(aluiSource,1,&aluiBuffer1);
    alDeleteBuffers(1,&aluiBuffer1);
    alDeleteBuffers(1,&aluiBuffer2);
    alDeleteSources(1,&aluiSource);
    ov_clear(&oggFile);
    return false;
  }
  else if(bytes > 0)
    alSourceQueueBuffers(aluiSource,1,&aluiBuffer2);

  bDataLoaded = true;
  return true;
}

void SoundOggStream::Close(){
  int queued=0;
  ALuint Buffer;

  Stop();

  alGetSourcei(aluiSource,AL_BUFFERS_QUEUED,&queued);
  while(queued--)
    alSourceUnqueueBuffers(aluiSource,1,&Buffer);

  alDeleteBuffers(1,&aluiBuffer1);
  alDeleteBuffers(1,&aluiBuffer2);
  alDeleteSources(1,&aluiSource);
  bDataLoaded = false;

  ov_clear(&oggFile);
}

SoundOggStream::~SoundOggStream(){
  Close();
}

int SoundOggStream::Stream(ALuint Buffer){
    char data[BUFFER_SIZE];
    int  size = 0;
    int  section;
    int  result;

    while(size < BUFFER_SIZE)
    {
        result = ov_read(&oggFile, data + size, BUFFER_SIZE - size, 0, 2, 1, &section);

        if(result > 0)
          size += result;
        else if(result < 0)
          return -1;
        else
          break;
    }

    if(size != 0)
      alBufferData(Buffer, format, data, size, freq);

    return size;
}

bool SoundOggStream::Update(){
  int processed=0,bytes;
  ALuint Buffer;
  alGetSourcei(aluiSource,AL_BUFFERS_PROCESSED,&processed);
  while(processed--){
    alSourceUnqueueBuffers(aluiSource,1,&Buffer);

    bytes = Stream(Buffer);
    if( bytes < 0)
       return false;
    else if( bytes == 0){  //Ende des Streams?
      if(bRepeat){ //Stream wiederholen
        ov_pcm_seek(&oggFile,0);
        bytes = Stream(Buffer);
        if(bytes <= 0) //Wiederholen fehlgeschlagen
          return false;
      }
      else //Nicht wiederholen
        break;
    }

    alSourceQueueBuffers(aluiSource,1,&Buffer);
  }

  return true;
}

void SoundOggStream::SetRepeat(bool value){
  bRepeat = value;
}

//Listener

void Listener::Call(){
  vec4 temp,q;
  if(ReCalc){
    Dir = DirectionV4(Pos1,Pos2);
    Up.x = 0; Up.y = 0; Up.z = 1.0f; Up.w=1.0f;
    if(SkalarProductV4(Up,Dir)==1){
      Up.x = 1.0f;
      Up.y = 0.0f;
      Up.z = 0.0f;
    }
    else{
      temp = CrossProductV4(Up,Dir);
      temp = NormalizeV4(temp);
      Up = CrossProductV4(Dir,temp);
    }
    //Quaterion Zeugs
    if(Rotation != 0){
      q = CreateRotationQ(Dir,Rotation);
      q = NormalizeQ(q);
      Up = Mul(QToMat4a(q),Up);
    }
  }
  gluLookAt(Pos1.x,Pos1.y,Pos1.z,Pos2.x,Pos2.y,Pos2.z,Up.x,Up.y,Up.z);
  glGetFloatv(GL_MODELVIEW_MATRIX, CameraMatrix);

  //Sound Stuff
  float Orientation[6];
  memcpy(Orientation,Dir.f,3*sizeof(float));
  memcpy(&Orientation[3],Up.f,3*sizeof(float));

  alListenerfv(AL_POSITION,Pos1.f);
  alListenerfv(AL_ORIENTATION,Orientation);
}

void Listener::SetGain(float value){
  alListenerf(AL_GAIN,value);
}

void Listener::SetVelocity(float* array){
  alListenerfv(AL_VELOCITY,array);
}
