/*************************************************************
Copyright (c) 2008

Benjamin Thaut. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
     and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
     and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY BENJAMIN THAUT "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

web: www.benjamin-thaut.de
*************************************************************/
#include "spline.h"

CubicSpline::CubicSpline(){
  i2 = NULL;
  bValid = false;
}

CubicSpline::~CubicSpline(){
  if(i2 != NULL)
    delete i2;
}

void CubicSpline::AddControlPoint(vec4& ref){
  array.push_back(ref);
  bValid = false;
}

void CubicSpline::AddControlPoint(float x, float y, float z, float t){
  array.push_back(vec4(x,y,z,t));
  bValid = false;
}

void CubicSpline::InsertControlPoint(int insert_pos, vec4& ref){
  array.insert(array.begin() + insert_pos, ref);
  bValid = false;
}

void CubicSpline::DeleteControlPoint(int index){
  array.erase(array.begin()+index);
  bValid = false;
}

void CubicSpline::Update(){
  int n = array.size();
  if(i2 != NULL)
    delete i2;

  vec4 *u= new vec4[n];
  i2 = new vec4[n];
  vec4 p,qn,un;
  float sig;

  i2[0] = vec4(0.0f,0.0f,0.0f,0.0f);
  u[0] = vec4(0.0f,0.0f,0.0f,0.0f);

  for(int i=1;i<n-1;i++){
    sig=(array[i].w - array[i-1].w)/(array[i+1].w-array[i-1].w);
    p = i2[i-1]*sig+vec4(2.0f,2.0f,2.0f,0.0f);
    i2[i]=vec4(1.0,1.0,1.0,0.0)/p * (sig-1.0);
    u[i]=(array[i+1]-array[i])/(array[i+1].w - array[i].w) - (array[i]-array[i-1])/(array[i].w - array[i-1].w);
    u[i]=((u[i] * 6.0f)/(array[i+1].w-array[i-1].w)-u[i-1]*sig)/p;
  }

  qn = un = vec4(0.0,0.0,0.0,0.0);
  i2[n-1]=(un-qn*u[n-2])/(qn*i2[n-2]+vec4(1.0f,1.0f,1.0f,0.0f));
  for(int k=n-2;k>=0;k--)
    i2[k]=i2[k]*i2[k+1]+u[k];

  delete []u;
  bValid = true;
}

vec4 CubicSpline::Get(float t){
  if(!bValid)
    return vec4(0.0,0.0,0.0,1.0);
  if(t <= 0)
    return array[0];
  if(t >= array[array.size()-1].w)
    return array[array.size()-1];

  int klo,khi,k,n=array.size();
  float h,b,a;

  klo = 0;
  khi = n-1;
  while(khi - klo > 1){
    k = (khi+klo) >> 1;
    if(array[k].w > t)
      khi = k;
    else
      klo = k;
  }
  h = array[khi].w-array[klo].w;
  if(h == 0.0)
    return array[array.size()-1];

  a=(array[khi].w-t)/h;
  b=(t-array[klo].w)/h;
  return array[klo]*a + array[khi] * b + (i2[klo] * (a*a*a-a) + i2[khi] * (b*b*b-b))*(h*h)/6.0;
}
